@@ -191,3 +191,16 @@ def topk(k, x):
 def arange(start, stop, step, length, dtype):
     res = np.arange(start, stop, step, dtype)
     return res[:-1] if len(res) > length else res
+
+
+def astype(x, astype_dtype=None, **kwargs):
+    return x.astype(astype_dtype, **kwargs)
+
+
+def view(x, dtype, order='C'):
+    if order == 'C':
+        x = np.ascontiguousarray(x)
+        return x.view(dtype)
+    else:
+        x = np.asfortranarray(x)
+        return x.T.view(dtype).T
