@@ -28,12 +28,13 @@ except AttributeError:
 try:
     with warnings.catch_warnings():
         if (not np.allclose(np.divide(.4, 1, casting="unsafe"),
-                            np.divide(.4, 1, casting="unsafe", dtype=np.float))
-                or not np.allclose(np.divide(1, .5, dtype='i8'), 2)
-                or not np.allclose(np.divide(.4, 1), .4)):
+                            np.divide(.4, 1, casting="unsafe", dtype=np.float)) or
+            not np.allclose(np.divide(1, .5, dtype='i8'), 2) or
+           not np.allclose(np.divide(.4, 1), .4)):
             raise TypeError('Divide not working with dtype: '
                             'https://github.com/numpy/numpy/issues/3484')
         divide = np.divide
+        ma_divide = np.ma.divide
 
 except TypeError:
     # Divide with dtype doesn't work on Python 3
@@ -47,6 +48,10 @@ except TypeError:
             x = x.astype(dtype)
         return x
 
+    ma_divide = np.ma.core._DomainedBinaryOperation(divide,
+                                                    np.ma.core._DomainSafeDivide(),
+                                                    0, 1)
+
 # functions copied from numpy
 try:
     from numpy import broadcast_to, nanprod, nancumsum, nancumprod
