@@ -93,13 +93,6 @@ def test_keepdims_wrapper_two_axes():
     assert (rwf == np.array([[60, 92, 124]])).all()
 
 
-def eq(a, b):
-    c = a == b
-    if isinstance(c, np.ndarray):
-        c = c.all()
-    return c
-
-
 def test_coarsen():
     x = np.random.randint(10, size=(24, 24))
     y = coarsen(np.sum, x, {0: 2, 1: 4})
