@@ -9,16 +9,7 @@ from dask.array.ghost import (fractional_slice, getitem, trim_internal,
                               ghost_internal, nearest, constant, boundaries,
                               reflect, periodic, ghost)
 from dask.core import get
-from dask.array.utils import assert_eq
-
-
-def same_keys(a, b):
-    def key(k):
-        if isinstance(k, str):
-            return (k, -1, -1, -1)
-        else:
-            return k
-    return sorted(a.dask, key=key) == sorted(b.dask, key=key)
+from dask.array.utils import assert_eq, same_keys
 
 
 def test_fractional_slice():
@@ -362,3 +353,13 @@ def test_none_boundaries():
          [33, 33,  8,  9, 10, 11, 33, 33],
          [33, 33, 12, 13, 14, 15, 33, 33]])
     assert_eq(exp, res)
+
+
+def test_ghost_small():
+    x = da.ones((10, 10), chunks=(5, 5))
+
+    y = x.map_overlap(lambda x: x, depth=1)
+    assert len(y.dask) < 200
+
+    y = x.map_overlap(lambda x: x, depth=1, boundary='none')
+    assert len(y.dask) < 100
