@@ -1,19 +1,10 @@
 import pytest
 pytest.importorskip('numpy')
 
-from dask.utils import skip
-from dask.array.utils import assert_eq
-import dask.array as da
 import numpy as np
 
-
-def same_keys(a, b):
-    def key(k):
-        if isinstance(k, str):
-            return (k, -1, -1, -1)
-        else:
-            return k
-    return sorted(a.dask, key=key) == sorted(b.dask, key=key)
+import dask.array as da
+from dask.array.utils import assert_eq, same_keys
 
 
 def test_percentile():
@@ -37,7 +28,7 @@ def test_percentile():
               np.array(['a', 'd', 'e'], dtype=x.dtype))
 
 
-@skip
+@pytest.mark.skip
 def test_percentile_with_categoricals():
     try:
         import pandas as pd
