@@ -2,9 +2,11 @@ import pytest
 pytest.importorskip('numpy')
 
 import numpy as np
+
+import dask.array as da
 from dask.array.core import Array
 from dask.array.random import random, exponential, normal
-import dask.array as da
+from dask.array.utils import assert_eq
 from dask.multiprocessing import get as mpget
 from dask.multiprocessing import _dumps, _loads
 
@@ -12,11 +14,11 @@ from dask.multiprocessing import _dumps, _loads
 def test_RandomState():
     state = da.random.RandomState(5)
     x = state.normal(10, 1, size=10, chunks=5)
-    assert (x.compute() == x.compute()).all()
+    assert_eq(x, x)
 
     state = da.random.RandomState(5)
     y = state.normal(10, 1, size=10, chunks=5)
-    assert (x.compute() == y.compute()).all()
+    assert_eq(x, y)
 
 
 def test_concurrency():
@@ -38,24 +40,24 @@ def test_serializability():
 
     y = _loads(_dumps(x))
 
-    assert (x.compute() == y.compute()).all()
+    assert_eq(x, y)
 
 
 def test_determinisim_through_dask_values():
     samples_1 = da.random.RandomState(42).normal(size=1000, chunks=10)
     samples_2 = da.random.RandomState(42).normal(size=1000, chunks=10)
 
-    assert [v for k, v in sorted(samples_1.dask.items())] ==\
-           [v for k, v in sorted(samples_2.dask.items())]
+    assert set(samples_1.dask) == set(samples_2.dask)
+    assert_eq(samples_1, samples_2)
 
 
 def test_randomstate_consistent_names():
     state1 = da.random.RandomState(42)
     state2 = da.random.RandomState(42)
-    assert sorted(state1.normal(size=(100, 100), chunks=(10, 10)).dask) ==\
-           sorted(state2.normal(size=(100, 100), chunks=(10, 10)).dask)
-    assert sorted(state1.normal(size=100, loc=4.5, scale=5.0, chunks=10).dask) ==\
-           sorted(state2.normal(size=100, loc=4.5, scale=5.0, chunks=10).dask)
+    assert (sorted(state1.normal(size=(100, 100), chunks=(10, 10)).dask) ==
+            sorted(state2.normal(size=(100, 100), chunks=(10, 10)).dask))
+    assert (sorted(state1.normal(size=100, loc=4.5, scale=5.0, chunks=10).dask) ==
+            sorted(state2.normal(size=100, loc=4.5, scale=5.0, chunks=10).dask))
 
 
 def test_random():
@@ -116,8 +118,16 @@ def test_random_seed():
     a = da.random.normal(size=10, chunks=5)
     b = da.random.normal(size=10, chunks=5)
 
-    assert (x.compute() == a.compute()).all()
-    assert (y.compute() == b.compute()).all()
+    assert_eq(x, a)
+    assert_eq(y, b)
+
+
+def test_consistent_across_sizes():
+    x1 = da.random.RandomState(123).random(20, chunks=20)
+    x2 = da.random.RandomState(123).random(100, chunks=20)[:20]
+    x3 = da.random.RandomState(123).random(200, chunks=20)[:20]
+    assert_eq(x1, x2)
+    assert_eq(x1, x3)
 
 
 def test_random_all():
@@ -126,7 +136,7 @@ def test_random_all():
     da.random.chisquare(1, size=5, chunks=3).compute()
     da.random.exponential(1, size=5, chunks=3).compute()
     da.random.f(1, 2, size=5, chunks=3).compute()
-    da.random.gamma(5, 1, chunks=3).compute()
+    da.random.gamma(5, 1, size=5, chunks=3).compute()
     da.random.geometric(1, size=5, chunks=3).compute()
     da.random.gumbel(1, size=5, chunks=3).compute()
     da.random.hypergeometric(1, 2, 3, size=5, chunks=3).compute()
@@ -206,3 +216,52 @@ def test_multinomial():
         y = np.random.multinomial(20, [1 / 6.] * 6, size=size)
 
         assert x.shape == y.shape == x.compute().shape
+
+
+def test_choice():
+    np_dtype = np.random.choice(1, size=()).dtype
+    size = (10, 3)
+    chunks = 4
+    x = da.random.choice(3, size=size, chunks=chunks)
+    assert x.dtype == np_dtype
+    assert x.shape == size
+    res = x.compute()
+    assert res.dtype == np_dtype
+    assert res.shape == size
+
+    np_a = np.array([1, 3, 5, 7, 9], dtype='f8')
+    da_a = da.from_array(np_a, chunks=2)
+
+    for a in [np_a, da_a]:
+        x = da.random.choice(a, size=size, chunks=chunks)
+        res = x.compute()
+        assert x.dtype == np_a.dtype
+        assert res.dtype == np_a.dtype
+        assert set(np.unique(res)).issubset(np_a)
+
+    np_p = np.array([0, 0.2, 0.2, 0.3, 0.3])
+    da_p = da.from_array(np_p, chunks=2)
+
+    for a, p in [(da_a, np_p), (np_a, da_p)]:
+        x = da.random.choice(a, size=size, chunks=chunks, p=p)
+        res = x.compute()
+        assert x.dtype == np_a.dtype
+        assert res.dtype == np_a.dtype
+        assert set(np.unique(res)).issubset(np_a[1:])
+
+    np_dtype = np.random.choice(1, size=(), p=np.array([1])).dtype
+    x = da.random.choice(5, size=size, chunks=chunks, p=np_p)
+    res = x.compute()
+    assert x.dtype == np_dtype
+    assert res.dtype == np_dtype
+
+    errs = [(-1, None),             # negative a
+            (np_a[:, None], None),  # a must be 1D
+            (np_a, np_p[:, None]),  # p must be 1D
+            (np_a, np_p[:-2]),      # a and p must match
+            (3, np_p),              # a and p must match
+            (4, [0.2, 0.2, 0.3])]   # p must sum to 1
+
+    for (a, p) in errs:
+        with pytest.raises(ValueError):
+            da.random.choice(a, size=size, chunks=chunks, p=p)
