@@ -35,7 +35,7 @@ def test_full():
     a = da.full((3, 3), 100, chunks=(2, 2), dtype='i8')
 
     assert (a.compute() == 100).all()
-    assert a._dtype == a.compute(get=dask.get).dtype == 'i8'
+    assert a.dtype == a.compute(get=dask.get).dtype == 'i8'
 
 
 def test_can_make_really_big_array_of_ones():
