@@ -1,8 +1,8 @@
 from __future__ import absolute_import, division, print_function
 
 from .core import (Bag, Item, from_sequence, from_url, to_textfiles, concat,
-                   from_castra, from_delayed, bag_range as range,
-                   bag_zip as zip)
+                   from_delayed, map_partitions, bag_range as range,
+                   bag_zip as zip, bag_map as map)
 from .text import read_text
 from ..context import set_options
 from ..base import compute
