@@ -24,18 +24,18 @@ try:
     if LooseVersion(cytoolz.__version__) > '0.7.3':
         from cytoolz import accumulate  # noqa: F811
         _implement_accumulate = True
-except:
+except ImportError:
     from toolz import (frequencies, merge_with, join, reduceby,
                        count, pluck, groupby, topk)
 
-from ..base import Base, normalize_token, tokenize
+from ..base import Base, tokenize, dont_optimize, is_dask_collection
 from ..bytes.core import write_bytes
 from ..compatibility import apply, urlopen
-from ..context import _globals, defer_to_globals
+from ..context import _globals, globalmethod
 from ..core import quote, istask, get_dependencies, reverse_dict
 from ..delayed import Delayed
 from ..multiprocessing import get as mpget
-from ..optimize import fuse, cull, inline, dont_optimize
+from ..optimize import fuse, cull, inline
 from ..utils import (system_encoding, takes_multiple_arguments, funcname,
                      digit, insert, ensure_dict)
 
@@ -48,7 +48,7 @@ no_result = type('no_result', (object,),
 
 def lazify_task(task, start=True):
     """
-    Given a task, remove unnecessary calls to ``list`` and ``reify``
+    Given a task, remove unnecessary calls to ``list`` and ``reify``.
 
     This traverses tasks and small lists.  We choose not to traverse down lists
     of size >= 50 because it is unlikely that sequences this long contain other
@@ -74,7 +74,7 @@ def lazify_task(task, start=True):
 
 def lazify(dsk):
     """
-    Remove unnecessary calls to ``list`` in tasks
+    Remove unnecessary calls to ``list`` in tasks.
 
     See Also
     --------
@@ -84,14 +84,14 @@ def lazify(dsk):
 
 
 def inline_singleton_lists(dsk, dependencies=None):
-    """ Inline lists that are only used once
+    """ Inline lists that are only used once.
 
     >>> d = {'b': (list, 'a'),
     ...      'c': (f, 'b', 1)}     # doctest: +SKIP
     >>> inline_singleton_lists(d)  # doctest: +SKIP
     {'c': (f, (list, 'a'), 1)}
 
-    Pairs nicely with lazify afterwards
+    Pairs nicely with lazify afterwards.
     """
     if dependencies is None:
         dependencies = {k: get_dependencies(dsk, task=v)
@@ -106,9 +106,8 @@ def inline_singleton_lists(dsk, dependencies=None):
     return dsk
 
 
-@defer_to_globals('bag_optimize', falsey=dont_optimize)
 def optimize(dsk, keys, fuse_keys=None, rename_fused_keys=True, **kwargs):
-    """ Optimize a dask from a dask.bag """
+    """ Optimize a dask from a dask Bag. """
     dsk2, dependencies = cull(dsk, keys)
     dsk3, dependencies = fuse(dsk2, keys + (fuse_keys or []), dependencies,
                               rename_keys=rename_fused_keys)
@@ -120,7 +119,7 @@ def optimize(dsk, keys, fuse_keys=None, rename_fused_keys=True, **kwargs):
 def to_textfiles(b, path, name_function=None, compression='infer',
                  encoding=system_encoding, compute=True, get=None,
                  storage_options=None):
-    """ Write bag to disk, one filename per partition, one line per element
+    """ Write dask Bag to disk, one filename per partition, one line per element.
 
     **Paths**: This will create one file for each partition in your bag. You
     can specify the filenames in a variety of ways.
@@ -174,17 +173,18 @@ def to_textfiles(b, path, name_function=None, compression='infer',
     >>> b_dict.map(json.dumps).to_textfiles("/path/to/data/*.json")  # doctest: +SKIP
     """
     from dask import delayed
-    writes = write_bytes(b.to_delayed(), path, name_function, compression,
-                         encoding=encoding, **(storage_options or {}))
+    (writes,names) = write_bytes(b.to_delayed(), path, name_function, compression,
+                                 encoding=encoding, **(storage_options or {}))
 
     # Use Bag optimizations on these delayed objects
     dsk = ensure_dict(delayed(writes).dask)
-    dsk2 = Bag._optimize(dsk, [w.key for w in writes])
+    dsk2 = Bag.__dask_optimize__(dsk, [w.key for w in writes])
     out = [Delayed(w.key, dsk2) for w in writes]
 
     if compute:
-        get = get or _globals.get('get', None) or Bag._default_get
+        get = get or _globals.get('get', None) or Bag.__dask_scheduler__
         delayed(out).compute(get=get)
+        return names
     else:
         return out
 
@@ -242,7 +242,7 @@ class StringAccessor(object):
                 raise
 
     def match(self, pattern):
-        """ Filter strings by those that match a pattern
+        """ Filter strings by those that match a pattern.
 
         Examples
         --------
@@ -261,7 +261,7 @@ class StringAccessor(object):
 
 
 def robust_wraps(wrapper):
-    """ A weak version of wraps that only copies doc """
+    """ A weak version of wraps that only copies doc. """
     def _(wrapped):
         wrapped.__doc__ = wrapper.__doc__
         return wrapped
@@ -269,13 +269,33 @@ def robust_wraps(wrapper):
 
 
 class Item(Base):
-    _optimize = staticmethod(optimize)
-    _default_get = staticmethod(mpget)
-    _finalize = staticmethod(finalize_item)
+    def __init__(self, dsk, key):
+        self.dask = dsk
+        self.key = key
+        self.name = key
+
+    def __dask_graph__(self):
+        return self.dask
+
+    def __dask_keys__(self):
+        return [self.key]
+
+    def __dask_tokenize__(self):
+        return self.key
+
+    __dask_optimize__ = globalmethod(optimize, key='bag_optimize',
+                                     falsey=dont_optimize)
+    __dask_scheduler__ = staticmethod(mpget)
+
+    def __dask_postcompute__(self):
+        return finalize_item, ()
+
+    def __dask_postpersist__(self):
+        return Item, (self.key,)
 
     @staticmethod
     def from_delayed(value):
-        """ Create bag item from a dask.delayed value
+        """ Create bag item from a dask.delayed value.
 
         See ``dask.bag.from_delayed`` for details
         """
@@ -285,11 +305,6 @@ class Item(Base):
         assert isinstance(value, Delayed)
         return Item(ensure_dict(value.dask), value.key)
 
-    def __init__(self, dsk, key):
-        self.dask = dsk
-        self.key = key
-        self.name = key
-
     @property
     def _args(self):
         return (self.dask, self.key)
@@ -300,9 +315,6 @@ class Item(Base):
     def __setstate__(self, state):
         self.dask, self.key = state
 
-    def _keys(self):
-        return [self.key]
-
     def apply(self, func):
         name = 'apply-{0}-{1}'.format(funcname(func), tokenize(self, func))
         dsk = {name: (func, self.key)}
@@ -311,12 +323,13 @@ class Item(Base):
     __int__ = __float__ = __complex__ = __bool__ = Base.compute
 
     def to_delayed(self):
-        """ Convert bag item to dask Delayed
+        """ Convert bag item to dask.delayed.
 
         Returns a single value.
         """
         from dask.delayed import Delayed
-        dsk = self._optimize(self.dask, [self.key])
+        dsk = self.__dask_optimize__(self.__dask_graph__(),
+                                     self.__dask_keys__())
         return Delayed(self.key, dsk)
 
 
@@ -349,15 +362,30 @@ class Bag(Base):
     >>> int(b.fold(lambda x, y: x + y))  # doctest: +SKIP
     30
     """
-    _optimize = staticmethod(optimize)
-    _default_get = staticmethod(mpget)
-    _finalize = staticmethod(finalize)
-
     def __init__(self, dsk, name, npartitions):
         self.dask = dsk
         self.name = name
         self.npartitions = npartitions
 
+    def __dask_graph__(self):
+        return self.dask
+
+    def __dask_keys__(self):
+        return [(self.name, i) for i in range(self.npartitions)]
+
+    def __dask_tokenize__(self):
+        return self.name
+
+    __dask_optimize__ = globalmethod(optimize, key='bag_optimize',
+                                     falsey=dont_optimize)
+    __dask_scheduler__ = staticmethod(mpget)
+
+    def __dask_postcompute__(self):
+        return finalize, ()
+
+    def __dask_postpersist__(self):
+        return type(self), (self.name, self.npartitions)
+
     def __str__(self):
         name = self.name if len(self.name) < 10 else self.name[:7] + '...'
         return 'dask.bag<%s, npartitions=%d>' % (name, self.npartitions)
@@ -425,13 +453,7 @@ class Bag(Base):
         >>> b.map(myadd, b.max()).compute()
         [4, 5, 6, 7, 8]
         """
-        if takes_multiple_arguments(func, varargs=False) and not args:
-            warn("Automatic 'splatting' of arguments in `Bag.map` is "
-                 "deprecated and will be removed in a future release. "
-                 "Please use `Bag.starmap` instead.")
-            return self.starmap(func, **kwargs)
-        else:
-            return bag_map(func, self, *args, **kwargs)
+        return bag_map(func, self, *args, **kwargs)
 
     def starmap(self, func, **kwargs):
         """Apply a function using argument tuples from the given bag.
@@ -499,7 +521,7 @@ class Bag(Base):
         self.dask, self.name, self.npartitions = state
 
     def filter(self, predicate):
-        """ Filter elements in collection by a predicate function
+        """ Filter elements in collection by a predicate function.
 
         >>> def iseven(x):
         ...     return x % 2 == 0
@@ -548,7 +570,7 @@ class Bag(Base):
         return type(self)(merge(self.dask, dsk), name, self.npartitions)
 
     def remove(self, predicate):
-        """ Remove elements in collection that match predicate
+        """ Remove elements in collection that match predicate.
 
         >>> def iseven(x):
         ...     return x % 2 == 0
@@ -604,7 +626,7 @@ class Bag(Base):
         return map_partitions(func, self, *args, **kwargs)
 
     def pluck(self, key, default=no_default):
-        """ Select item from all tuples/dicts in collection
+        """ Select item from all tuples/dicts in collection.
 
         >>> b = from_sequence([{'name': 'Alice', 'credits': [1, 2, 3]},
         ...                    {'name': 'Bob',   'credits': [10, 20]}])
@@ -705,7 +727,7 @@ class Bag(Base):
                                   split_every=split_every)
 
     def frequencies(self, split_every=None):
-        """ Count number of occurrences of each distinct element
+        """ Count number of occurrences of each distinct element.
 
         >>> b = from_sequence(['Alice', 'Bob', 'Alice'])
         >>> dict(b.frequencies())  # doctest: +SKIP
@@ -750,7 +772,7 @@ class Bag(Base):
 
     def reduction(self, perpartition, aggregate, split_every=None,
                   out_type=Item, name=None):
-        """ Reduce collection with reduction operators
+        """ Reduce collection with reduction operators.
 
         Parameters
         ----------
@@ -827,7 +849,7 @@ class Bag(Base):
         return self.reduction(all, all, split_every=split_every)
 
     def count(self, split_every=None):
-        """ Count the number of elements """
+        """ Count the number of elements. """
         return self.reduction(count, sum, split_every=split_every)
 
     def mean(self):
@@ -868,7 +890,7 @@ class Bag(Base):
         return self.var(ddof=ddof).apply(math.sqrt)
 
     def join(self, other, on_self, on_other=None):
-        """ Join collection with another collection
+        """ Joins collection with another collection.
 
         Other collection must be an Iterable, and not a Bag.
 
@@ -888,7 +910,7 @@ class Bag(Base):
         return type(self)(merge(self.dask, dsk), name, self.npartitions)
 
     def product(self, other):
-        """ Cartesian product between two bags """
+        """ Cartesian product between two bags. """
         assert isinstance(other, Bag)
         name = 'product-' + tokenize(self, other)
         n, m = self.npartitions, other.npartitions
@@ -899,8 +921,8 @@ class Bag(Base):
         return type(self)(merge(self.dask, other.dask, dsk), name, n * m)
 
     def foldby(self, key, binop, initial=no_default, combine=None,
-               combine_initial=no_default):
-        """ Combined reduction and groupby
+               combine_initial=no_default, split_every=None):
+        """ Combined reduction and groupby.
 
         Foldby provides a combined groupby and reduce for efficient parallel
         split-apply-combine tasks.
@@ -962,6 +984,11 @@ class Bag(Base):
         like above this is often ``0`` or the identity element for your
         operation.
 
+        **split_every**
+
+        Group partitions into groups of this size while performing reduction.
+        Defaults to 8.
+
         >>> b.foldby('name', binop, 0, combine, 0)  # doctest: +SKIP
 
         See Also
@@ -970,33 +997,61 @@ class Bag(Base):
         toolz.reduceby
         pyspark.combineByKey
         """
+        if split_every is None:
+            split_every = 8
+        if split_every is False:
+            split_every = self.npartitions
+
         token = tokenize(self, key, binop, initial, combine, combine_initial)
         a = 'foldby-a-' + token
-        b = 'foldby-b-' + token
         if combine is None:
             combine = binop
         if initial is not no_default:
-            dsk = dict(((a, i), (reduceby, key, binop, (self.name, i), initial))
-                       for i in range(self.npartitions))
+            dsk = {(a, i): (reduceby, key, binop, (self.name, i), initial)
+                   for i in range(self.npartitions)}
         else:
-            dsk = dict(((a, i), (reduceby, key, binop, (self.name, i)))
-                       for i in range(self.npartitions))
+            dsk = {(a, i): (reduceby, key, binop, (self.name, i))
+                   for i in range(self.npartitions)}
 
         def combine2(acc, x):
             return combine(acc, x[1])
 
+        depth = 0
+        k = self.npartitions
+        b = a
+        while k > split_every:
+            c = b + str(depth)
+            if combine_initial is not no_default:
+                dsk2 = {(c, i): (reduceby, 0, combine2,
+                                 (toolz.concat, (map, dictitems,
+                                                 [(b, j) for j in inds])),
+                                 combine_initial)
+                        for i, inds in enumerate(partition_all(split_every,
+                                                               range(k)))}
+            else:
+                dsk2 = {(c, i): (merge_with, (partial, reduce, combine),
+                                 [(b, j) for j in inds])
+                        for i, inds in enumerate(partition_all(split_every,
+                                                               range(k)))}
+            dsk.update(dsk2)
+            k = len(dsk2)
+            b = c
+            depth += 1
+
+        e = 'foldby-b-' + token
         if combine_initial is not no_default:
-            dsk2 = {(b, 0): (dictitems, (reduceby, 0, combine2,
-                                         (toolz.concat, (map, dictitems,
-                                                         list(dsk.keys()))),
-                                         combine_initial))}
+            dsk[(e, 0)] = (dictitems, (reduceby, 0, combine2,
+                                       (toolz.concat, (map, dictitems,
+                                                       [(b, j) for j in range(k)])),
+                                       combine_initial))
         else:
-            dsk2 = {(b, 0): (dictitems, (merge_with, (partial, reduce, combine),
-                                         list(dsk.keys())))}
-        return type(self)(merge(self.dask, dsk, dsk2), b, 1)
+            dsk[(e, 0)] = (dictitems, (merge_with, (partial, reduce, combine),
+                                       [(b, j) for j in range(k)]))
+
+        return type(self)(merge(self.dask, dsk), e, 1)
 
     def take(self, k, npartitions=1, compute=True):
-        """ Take the first k elements
+        """ Take the first k elements.
 
         Parameters
         ----------
@@ -1043,11 +1098,8 @@ class Bag(Base):
         else:
             return b
 
-    def _keys(self):
-        return [(self.name, i) for i in range(self.npartitions)]
-
     def flatten(self):
-        """ Concatenate nested lists into one long list
+        """ Concatenate nested lists into one long list.
 
         >>> b = from_sequence([[1], [2, 3]])
         >>> list(b)
@@ -1061,10 +1113,6 @@ class Bag(Base):
                    for i in range(self.npartitions))
         return type(self)(merge(self.dask, dsk), name, self.npartitions)
 
-    def concat(self):
-        warn("Deprecated.  Use the .flatten method instead")
-        return self.flatten()
-
     def __iter__(self):
         return iter(self.compute())
 
@@ -1120,7 +1168,7 @@ class Bag(Base):
             raise NotImplementedError(msg)
 
     def to_dataframe(self, meta=None, columns=None):
-        """ Create Dask Dataframe from a Dask Bag
+        """ Create Dask Dataframe from a Dask Bag.
 
         Bag should contain tuples, dict records, or scalars.
 
@@ -1182,24 +1230,24 @@ class Bag(Base):
         cols = list(meta.columns)
         dtypes = meta.dtypes.to_dict()
         name = 'to_dataframe-' + tokenize(self, cols, dtypes)
-        dsk = {(name, i): (to_dataframe, (self.name, i), cols, dtypes)
-               for i in range(self.npartitions)}
-
+        dsk = self.__dask_optimize__(self.dask, self.__dask_keys__())
+        dsk.update({(name, i): (to_dataframe, (self.name, i), cols, dtypes)
+                    for i in range(self.npartitions)})
         divisions = [None] * (self.npartitions + 1)
-        return dd.DataFrame(merge(optimize(self.dask, self._keys()), dsk),
-                            name, meta, divisions)
+        return dd.DataFrame(dsk, name, meta, divisions)
 
     def to_delayed(self):
-        """ Convert bag to list of dask Delayed
+        """ Convert bag to list of dask Delayed.
 
         Returns list of Delayed, one per partition.
         """
         from dask.delayed import Delayed
-        dsk = self._optimize(self.dask, self._keys())
-        return [Delayed(k, dsk) for k in self._keys()]
+        keys = self.__dask_keys__()
+        dsk = self.__dask_optimize__(self.__dask_graph__(), keys)
+        return [Delayed(k, dsk) for k in keys]
 
     def repartition(self, npartitions):
-        """ Coalesce bag into fewer partitions
+        """ Coalesce bag into fewer partitions.
 
         Examples
         --------
@@ -1257,8 +1305,8 @@ class Bag(Base):
         Accumulate also takes an optional argument that will be used as the
         first value.
 
-        >>> b.accumulate(add, -1)  # doctest: +SKIP
-        [-1, 0, 2, 5, 9, 15]
+        >>> b.accumulate(add, initial=-1)  # doctest: +SKIP
+        [-1, 0, 2, 5, 9, 14]
         """
         if not _implement_accumulate:
             raise NotImplementedError("accumulate requires `toolz` > 0.7.4"
@@ -1288,12 +1336,8 @@ def accumulate_part(binop, seq, initial, is_first=False):
     return res[1:], res[-1]
 
 
-normalize_token.register(Item, lambda a: a.key)
-normalize_token.register(Bag, lambda a: a.name)
-
-
 def partition(grouper, sequence, npartitions, p, nelements=2**20):
-    """ Partition a bag along a grouper, store partitions on disk """
+    """ Partition a bag along a grouper, store partitions on disk. """
     for block in partition_all(nelements, sequence):
         d = groupby(grouper, block)
         d2 = defaultdict(list)
@@ -1304,13 +1348,13 @@ def partition(grouper, sequence, npartitions, p, nelements=2**20):
 
 
 def collect(grouper, group, p, barrier_token):
-    """ Collect partitions from disk and yield k,v group pairs """
+    """ Collect partitions from disk and yield k,v group pairs. """
     d = groupby(grouper, p.get(group, lock=False))
     return list(d.items())
 
 
 def from_sequence(seq, partition_size=None, npartitions=None):
-    """ Create dask from Python sequence
+    """ Create a dask Bag from Python sequence.
 
     This sequence should be relatively small in memory.  Dask Bag works
     best when it handles loading your data itself.  Commonly we load a
@@ -1334,7 +1378,7 @@ def from_sequence(seq, partition_size=None, npartitions=None):
 
     See Also
     --------
-    read_text: Create bag from textfiles
+    read_text: Create bag from text files
     """
     seq = list(seq)
     if npartitions and not partition_size:
@@ -1352,7 +1396,7 @@ def from_sequence(seq, partition_size=None, npartitions=None):
 
 
 def from_url(urls):
-    """Create a dask.bag from a url
+    """Create a dask Bag from a url.
 
     Examples
     --------
@@ -1361,14 +1405,14 @@ def from_url(urls):
     1
 
     >>> a.take(8)  # doctest: +SKIP
-    ('Dask\\n',
-     '====\\n',
-     '\\n',
-     '|Build Status| |Coverage| |Doc Status| |Gitter|\\n',
-     '\\n',
-     'Dask provides multi-core execution on larger-than-memory datasets using blocked\\n',
-     'algorithms and task scheduling.  It maps high-level NumPy and list operations\\n',
-     'on large datasets on to graphs of many operations on small in-memory datasets.\\n')
+    (b'Dask\\n',
+     b'====\\n',
+     b'\\n',
+     b'|Build Status| |Coverage| |Doc Status| |Gitter| |Version Status|\\n',
+     b'\\n',
+     b'Dask is a flexible parallel computing library for analytics.  See\\n',
+     b'documentation_ for more information.\\n',
+     b'\\n')
 
     >>> b = from_url(['http://github.com', 'http://google.com'])  # doctest: +SKIP
     >>> b.npartitions  # doctest: +SKIP
@@ -1393,7 +1437,7 @@ def dictitems(d):
 
 
 def concat(bags):
-    """ Concatenate many bags together, unioning all elements
+    """ Concatenate many bags together, unioning all elements.
 
     >>> import dask.bag as db
     >>> a = db.from_sequence([1, 2, 3])
@@ -1405,8 +1449,8 @@ def concat(bags):
     """
     name = 'concat-' + tokenize(*bags)
     counter = itertools.count(0)
-    dsk = dict(((name, next(counter)), key)
-               for bag in bags for key in sorted(bag._keys()))
+    dsk = {(name, next(counter)): key
+           for bag in bags for key in bag.__dask_keys__()}
     return Bag(merge(dsk, *[b.dask for b in bags]), name, len(dsk))
 
 
@@ -1419,7 +1463,7 @@ def reify(seq):
 
 
 def from_delayed(values):
-    """ Create bag from many dask.delayed objects
+    """ Create bag from many dask Delayed objects.
 
     These objects will become the partitions of the resulting Bag.  They should
     evaluate to a ``list`` or some other concrete sequence.
@@ -1597,7 +1641,7 @@ def starmap_chunk(f, x, kwargs):
 
 
 def unpack_scalar_dask_kwargs(kwargs):
-    """Extracts dask values from kwargs
+    """Extracts dask values from kwargs.
 
     Currently only ``dask.bag.Item`` and ``dask.delayed.Delayed`` are
     supported.  Returns a merged dask graph and a task resulting in a keyword
@@ -1609,7 +1653,7 @@ def unpack_scalar_dask_kwargs(kwargs):
         if isinstance(v, (Delayed, Item)):
             dsk.update(ensure_dict(v.dask))
             kwargs2[k] = v.key
-        elif isinstance(v, Base):
+        elif is_dask_collection(v):
             raise NotImplementedError("dask.bag doesn't support kwargs of "
                                       "type %s" % type(v).__name__)
         else:
@@ -1779,7 +1823,7 @@ def map_partitions(func, *args, **kwargs):
                 dsk.update(ensure_dict(a.dask))
                 if isinstance(a, Bag):
                     bags.append(a)
-            elif isinstance(a, Base):
+            elif is_dask_collection(a):
                 raise NotImplementedError("dask.bag doesn't support args of "
                                           "type %s" % type(a).__name__)
 
@@ -1952,7 +1996,7 @@ def safe_take(n, b):
 
 
 def random_sample(x, state_data, prob):
-    """Filter elements of `x` by a probability `prob`
+    """Filter elements of `x` by a probability `prob`.
 
     Parameters
     ----------
@@ -1970,7 +2014,7 @@ def random_sample(x, state_data, prob):
 
 
 def random_state_data_python(n, random_state=None):
-    """Return a list of tuples that can initialize
+    """Return a list of tuples that can initialize.
     ``random.Random``.
 
     Parameters
@@ -1989,7 +2033,7 @@ def random_state_data_python(n, random_state=None):
 
 
 def split(seq, n):
-    """ Split apart a sequence into n equal pieces
+    """ Split apart a sequence into n equal pieces.
 
     >>> split(range(10), 3)
     [[0, 1, 2], [3, 4, 5], [6, 7, 8, 9]]
