@@ -45,72 +45,7 @@ def test_Bag():
 
 
 def test_keys():
-    assert sorted(b._keys()) == sorted(dsk.keys())
-
-
-# Deprecated test for old map behavior
-def test_map():
-    c = b.map(inc)
-    assert c.compute() == list(map(inc, b.compute()))
-    assert c.name == b.map(inc).name
-
-
-# Deprecated test for old map behavior
-def test_map_function_with_multiple_arguments():
-    b = db.from_sequence([(1, 10), (2, 20), (3, 30)], npartitions=3)
-    assert list(b.map(lambda x, y: x + y).compute(get=dask.get)) == [11, 22, 33]
-    assert list(b.map(list).compute()) == [[1, 10], [2, 20], [3, 30]]
-
-
-class A(object):
-    def __init__(self, a, b, c):
-        pass
-
-
-class B(object):
-    def __init__(self, a):
-        pass
-
-
-# Deprecated test for old map behavior
-def test_map_with_constructors():
-    assert db.from_sequence([[1, 2, 3]]).map(A).compute()
-    assert db.from_sequence([1, 2, 3]).map(B).compute()
-    assert db.from_sequence([[1, 2, 3]]).map(B).compute()
-
-    failed = False
-    try:
-        db.from_sequence([[1,]]).map(A).compute()
-    except TypeError:
-        failed = True
-    assert failed
-
-
-# Deprecated test for old map behavior
-def test_map_with_builtins():
-    b = db.from_sequence(range(3))
-    assert ' '.join(b.map(str)) == '0 1 2'
-    assert b.map(str).map(tuple).compute() == [('0',), ('1',), ('2',)]
-    assert b.map(str).map(tuple).map(any).compute() == [True, True, True]
-
-    b2 = b.map(lambda n: [(n, n + 1), (2 * (n - 1), -n)])
-    assert b2.map(dict).compute() == [{0: 1, -2: 0}, {1: 2, 0: -1}, {2: -2}]
-    assert b.map(lambda n: (n, n + 1)).map(pow).compute() == [0, 1, 8]
-    assert b.map(bool).compute() == [False, True, True]
-    assert db.from_sequence([(1, 'real'), ('1', 'real')]).map(hasattr).compute() == \
-        [True, False]
-
-
-# Deprecated test for old map behavior
-def test_map_with_kwargs():
-    b = db.from_sequence(range(100), npartitions=10)
-    assert b.map(lambda x, factor=0: x * factor,
-                 factor=2).sum().compute() == 9900.0
-    assert b.map(lambda x, total=0: x / total,
-                 total=b.sum()).sum().compute() == 1.0
-    assert b.map(lambda x, factor=0, total=0: x * factor / total,
-                 total=b.sum(),
-                 factor=2).sum().compute() == 2.0
+    assert b.__dask_keys__() == sorted(dsk.keys())
 
 
 def test_bag_map():
@@ -435,6 +370,23 @@ def test_foldby():
     assert set(c) == set(reduceby(iseven, lambda acc, x: acc + x, L, 0).items())
 
 
+def test_foldby_tree_reduction():
+    dsk = list()
+    for n in [1, 7, 32]:
+        b = db.from_sequence(range(100), npartitions=n)
+        c = b.foldby(iseven, add)
+        dsk += [c]
+        for m in [False, None, 2, 3]:
+            d = b.foldby(iseven, add, split_every=m)
+            e = b.foldby(iseven, add, 0, split_every=m)
+            f = b.foldby(iseven, add, 0, add, split_every=m)
+            g = b.foldby(iseven, add, 0, add, 0, split_every=m)
+            dsk += [d,e,f,g]
+    results = dask.compute(dsk)
+    first = results[0]
+    assert all([r == first for r in results])
+
+
 def test_map_partitions():
     assert list(b.map_partitions(len)) == [5, 5, 5]
     assert b.map_partitions(len).name == b.map_partitions(len).name
@@ -763,7 +715,6 @@ def test_concat():
 def test_flatten():
     b = db.from_sequence([[1], [2, 3]])
     assert list(b.flatten()) == [1, 2, 3]
-    assert list(b.flatten()) == list(b.concat())
     assert b.flatten().name == b.flatten().name
 
 
@@ -1091,7 +1042,7 @@ def test_repartition(nin, nout):
 
     assert c.npartitions == nout
     assert b.compute(get=dask.get) == c.compute(get=dask.get)
-    results = dask.get(c.dask, c._keys())
+    results = dask.get(c.dask, c.__dask_keys__())
     assert all(results)
 
 
@@ -1127,7 +1078,7 @@ def test_accumulate():
 def test_groupby_tasks():
     b = db.from_sequence(range(160), npartitions=4)
     out = b.groupby(lambda x: x % 10, max_branch=4, method='tasks')
-    partitions = dask.get(out.dask, out._keys())
+    partitions = dask.get(out.dask, out.__dask_keys__())
 
     for a in partitions:
         for b in partitions:
@@ -1137,7 +1088,7 @@ def test_groupby_tasks():
     b = db.from_sequence(range(1000), npartitions=100)
     out = b.groupby(lambda x: x % 123, method='tasks')
     assert len(out.dask) < 100**2
-    partitions = dask.get(out.dask, out._keys())
+    partitions = dask.get(out.dask, out.__dask_keys__())
 
     for a in partitions:
         for b in partitions:
@@ -1146,7 +1097,7 @@ def test_groupby_tasks():
 
     b = db.from_sequence(range(10000), npartitions=345)
     out = b.groupby(lambda x: x % 2834, max_branch=24, method='tasks')
-    partitions = dask.get(out.dask, out._keys())
+    partitions = dask.get(out.dask, out.__dask_keys__())
 
     for a in partitions:
         for b in partitions:
@@ -1265,11 +1216,12 @@ def test_optimize_fuse_keys():
     y = x.map(inc)
     z = y.map(inc)
 
-    dsk = z._optimize(z.dask, z._keys())
+    dsk = z.__dask_optimize__(z.dask, z.__dask_keys__())
     assert not set(y.dask) & set(dsk)
 
-    dsk = z._optimize(z.dask, z._keys(), fuse_keys=y._keys())
-    assert all(k in dsk for k in y._keys())
+    dsk = z.__dask_optimize__(z.dask, z.__dask_keys__(),
+                              fuse_keys=y.__dask_keys__())
+    assert all(k in dsk for k in y.__dask_keys__())
 
 
 def test_reductions_are_lazy():
@@ -1297,13 +1249,13 @@ def test_repeated_groupby():
     assert valmap(len, dict(c)) == valmap(len, dict(c))
 
 
-def test_temporary_directory():
+def test_temporary_directory(tmpdir):
     b = db.range(10, npartitions=4)
 
-    with dask.set_options(temporary_directory=os.getcwd()):
+    with dask.set_options(temporary_directory=str(tmpdir)):
         b2 = b.groupby(lambda x: x % 2)
         b2.compute()
-        assert any(fn.endswith('.partd') for fn in os.listdir(os.getcwd()))
+        assert any(fn.endswith('.partd') for fn in os.listdir(str(tmpdir)))
 
 
 def test_empty_bag():
@@ -1312,3 +1264,10 @@ def test_empty_bag():
     assert not b.map(inc).any().compute(get=dask.get)
     assert not b.map(inc).sum().compute(get=dask.get)
     assert not b.map(inc).count().compute(get=dask.get)
+
+
+def test_bag_paths():
+    b = db.from_sequence(['abc', '123', 'xyz'], npartitions=2)
+    assert b.to_textfiles('foo*') == ['foo0', 'foo1']
+    os.remove('foo0')
+    os.remove('foo1')
