@@ -1,7 +1,7 @@
 from __future__ import absolute_import, division, print_function
 
+from abc import ABCMeta
 from collections import OrderedDict, Iterator
-import copy
 from functools import partial
 from hashlib import md5
 import inspect
@@ -9,28 +9,39 @@ import pickle
 import os
 import threading
 import uuid
+import warnings
 
 from toolz import merge, groupby, curry, identity
 from toolz.functoolz import Compose
 
-from . import sharedict
-from .compatibility import bind_method, long, unicode, PY3
-from .context import _globals
+from .compatibility import long, unicode
+from .context import _globals, thread_state
 from .core import flatten
 from .hashing import hash_buffer_hex
 from .utils import Dispatch, ensure_dict
-from .sharedict import ShareDict
-
-__all__ = ("Base", "compute", "normalize_token", "tokenize", "visualize")
 
 
-class Base(object):
-    """Base class for dask collections"""
+__all__ = ("DaskMethodsMixin",
+           "is_dask_collection",
+           "compute", "persist", "visualize",
+           "tokenize", "normalize_token")
+
+
+def is_dask_collection(x):
+    """Returns ``True`` if ``x`` is a dask collection"""
+    try:
+        return x.__dask_graph__() is not None
+    except (AttributeError, TypeError):
+        return False
+
+
+class DaskMethodsMixin(object):
+    """A mixin adding standard dask collection methods"""
+    __slots__ = ()
 
     def visualize(self, filename='mydask', format=None, optimize_graph=False,
                   **kwargs):
-        """
-        Render the computation of this object's task graph using graphviz.
+        """Render the computation of this object's task graph using graphviz.
 
         Requires ``graphviz`` to be installed.
 
@@ -53,7 +64,7 @@ class Base(object):
         result : IPython.diplay.Image, IPython.display.SVG, or None
             See dask.dot.dot_graph for more information.
 
-        See also
+        See Also
         --------
         dask.base.visualize
         dask.dot.dot_graph
@@ -68,15 +79,37 @@ class Base(object):
                          optimize_graph=optimize_graph, **kwargs)
 
     def persist(self, **kwargs):
-        """ Persist this dask collection into memory
+        """Persist this dask collection into memory
 
-        See ``dask.base.persist`` for full docstring
+        This turns a lazy Dask collection into a Dask collection with the same
+        metadata, but now with the results fully computed or actively computing
+        in the background.
+
+        Parameters
+        ----------
+        get : callable, optional
+            A scheduler ``get`` function to use. If not provided, the default
+            is to check the global settings first, and then fall back to
+            the collection defaults.
+        optimize_graph : bool, optional
+            If True [default], the graph is optimized before computation.
+            Otherwise the graph is run as is. This can be useful for debugging.
+        **kwargs
+            Extra keywords to forward to the scheduler ``get`` function.
+
+        Returns
+        -------
+        New dask collections backed by in-memory data
+
+        See Also
+        --------
+        dask.base.persist
         """
         (result,) = persist(self, **kwargs)
         return result
 
     def compute(self, **kwargs):
-        """ Compute this dask collection
+        """Compute this dask collection
 
         This turns a lazy Dask collection into its in-memory equivalent.
         For example a Dask.array turns into a NumPy array and a Dask.dataframe
@@ -94,49 +127,138 @@ class Base(object):
             Otherwise the graph is run as is. This can be useful for debugging.
         kwargs
             Extra keywords to forward to the scheduler ``get`` function.
+
+        See Also
+        --------
+        dask.base.compute
         """
         (result,) = compute(self, traverse=False, **kwargs)
         return result
 
-    @classmethod
-    def _get(cls, dsk, keys, get=None, **kwargs):
-        get = get or _globals['get'] or cls._default_get
-        dsk2 = optimization_function(cls)(ensure_dict(dsk), keys, **kwargs)
-        return get(dsk2, keys, **kwargs)
+
+def call_finalize(finalize, args, results):
+    return finalize(results, *args)
+
+
+def add_ABCMeta(cls):
+    """Use the metaclass ABCMeta for this class"""
+    return ABCMeta(cls.__name__, cls.__bases__, cls.__dict__.copy())
+
+
+# TODO: this class is deprecated and should be removed in a future release.
+@add_ABCMeta
+class Base(DaskMethodsMixin):
+    """DEPRECATED. The recommended way to create a custom dask object now is to
+    implement the dask collection interface (see the docs), and optionally
+    subclass from ``DaskMethodsMixin`` if desired.
+
+    See http://dask.pydata.org/en/latest/custom-collections.html for more
+    information"""
+    __slots__ = ()
 
     @classmethod
-    def _bind_operator(cls, op):
-        """ bind operator to this class """
-        name = op.__name__
+    def __subclasshook__(cls, other):
+        if cls is Base:
+            warnings.warn("DeprecationWarning: `dask.base.Base` is deprecated. "
+                          "To check if an object is a dask collection use "
+                          "dask.base.is_dask_collection.\n\nSee http://dask."
+                          "pydata.org/en/latest/custom-collections.html "
+                          " for more information")
+        return NotImplemented
 
-        if name.endswith('_'):
-            # for and_ and or_
-            name = name[:-1]
-        elif name == 'inv':
-            name = 'invert'
+    def __dask_graph__(self):
+        # We issue a deprecation warning for the whole class here, as any
+        # non-instance check usage will end up calling `__dask_graph__`.
+        warnings.warn("DeprecationWarning: `dask.base.Base` is deprecated. "
+                      "To create a custom dask object implement the dask "
+                      "collection interface, and optionally subclass from "
+                      "``DaskMethodsMixin`` if desired.\n\nSee http://dask."
+                      "pydata.org/en/latest/custom-collections.html "
+                      " for more information")
+        return self.dask
 
-        meth = '__{0}__'.format(name)
+    def _keys(self):
+        warnings.warn("DeprecationWarning: the `_keys` method is deprecated, "
+                      "use `__dask_keys__` instead")
+        return self.__dask_keys__()
 
-        if name in ('abs', 'invert', 'neg', 'pos'):
-            bind_method(cls, meth, cls._get_unary_operator(op))
+    @property
+    def _finalize(self):
+        warnings.warn("DeprecationWarning: the `_finalize` method is "
+                      "deprecated, use `__dask_postcompute__` instead")
+        f, args = self.__dask_postcompute__()
+        return partial(call_finalize, f, args) if args else f
+
+    @classmethod
+    def _optimize(cls, *args, **kwargs):
+        warnings.warn("DeprecationWarning: the `_optimize` method is "
+                      "deprecated, use `__dask_optimize__` instead")
+        return cls.__dask_optimize__(*args, **kwargs)
+
+    @classmethod
+    def _get(cls, dsk, keys, **kwargs):
+        warnings.warn("DeprecationWarning: the `_get` method is "
+                      "deprecated, use ``dask.base.compute_as_if_collection`` "
+                      "instead")
+        return compute_as_if_collection(cls, dsk, keys, **kwargs)
+
+
+def compute_as_if_collection(cls, dsk, keys, get=None, **kwargs):
+    """Compute a graph as if it were of type cls.
+
+    Allows for applying the same optimizations and default scheduler."""
+    get = get or _globals['get'] or cls.__dask_scheduler__
+    dsk2 = optimization_function(cls)(ensure_dict(dsk), keys, **kwargs)
+    return get(dsk2, keys, **kwargs)
+
+
+def dont_optimize(dsk, keys, **kwargs):
+    return dsk
+
+
+def optimization_function(x):
+    return getattr(x, '__dask_optimize__', dont_optimize)
+
+
+def collections_to_dsk(collections, optimize_graph=True, **kwargs):
+    """
+    Convert many collections into a single dask graph, after optimization
+    """
+    optimizations = (kwargs.pop('optimizations', None) or
+                     _globals.get('optimizations', []))
+
+    if optimize_graph:
+        groups = groupby(optimization_function, collections)
+        groups = {opt: _extract_graph_and_keys(val)
+                  for opt, val in groups.items()}
+
+        for opt in optimizations:
+            groups = {k: (opt(dsk, keys), keys)
+                      for k, (dsk, keys) in groups.items()}
+
+        dsk = merge(*(opt(dsk, keys, **kwargs)
+                      for opt, (dsk, keys) in groups.items()))
+    else:
+        dsk, _ = _extract_graph_and_keys(collections)
+
+    return dsk
+
+
+def _extract_graph_and_keys(vals):
+    """Given a list of dask vals, return a single graph and a list of keys such
+    that ``get(dsk, keys)`` is equivalent to ``[v.compute() v in vals]``."""
+    dsk = {}
+    keys = []
+    for v in vals:
+        d = v.__dask_graph__()
+        if hasattr(d, 'dicts'):
+            for dd in d.dicts.values():
+                dsk.update(dd)
         else:
-            bind_method(cls, meth, cls._get_binary_operator(op))
+            dsk.update(d)
+        keys.append(v.__dask_keys__())
 
-            if name in ('eq', 'gt', 'ge', 'lt', 'le', 'ne', 'getitem'):
-                return
-
-            rmeth = '__r{0}__'.format(name)
-            bind_method(cls, rmeth, cls._get_binary_operator(op, inv=True))
-
-    @classmethod
-    def _get_unary_operator(cls, op):
-        """ Must return a method used by unary operator """
-        raise NotImplementedError
-
-    @classmethod
-    def _get_binary_operator(cls, op, inv=False):
-        """ Must return a method used by binary operator """
-        raise NotImplementedError
+    return dsk, keys
 
 
 def compute(*args, **kwargs):
@@ -186,28 +308,32 @@ def compute(*args, **kwargs):
                      else a for a in args)
 
     optimize_graph = kwargs.pop('optimize_graph', True)
-    variables = [a for a in args if isinstance(a, Base)]
+    variables = [a for a in args if is_dask_collection(a)]
     if not variables:
         return args
 
     get = kwargs.pop('get', None) or _globals['get']
 
+    if get is None and getattr(thread_state, 'key', False):
+        from distributed.worker import get_worker
+        get = get_worker().client.get
+
     if not get:
-        get = variables[0]._default_get
-        if not all(a._default_get == get for a in variables):
+        get = variables[0].__dask_scheduler__
+        if not all(a.__dask_scheduler__ == get for a in variables):
             raise ValueError("Compute called on multiple collections with "
                              "differing default schedulers. Please specify a "
                              "scheduler `get` function using either "
                              "the `get` kwarg or globally with `set_options`.")
 
     dsk = collections_to_dsk(variables, optimize_graph, **kwargs)
-    keys = [var._keys() for var in variables]
+    keys = [var.__dask_keys__() for var in variables]
+    postcomputes = [a.__dask_postcompute__() if is_dask_collection(a)
+                    else (None, a) for a in args]
     results = get(dsk, keys, **kwargs)
-
     results_iter = iter(results)
-    return tuple(a if not isinstance(a, Base)
-                 else a._finalize(next(results_iter))
-                 for a in args)
+    return tuple(a if f is None else f(next(results_iter), *a)
+                 for f, a in postcomputes)
 
 
 def visualize(*args, **kwargs):
@@ -238,7 +364,7 @@ def visualize(*args, **kwargs):
     result : IPython.diplay.Image, IPython.display.SVG, or None
         See dask.dot.dot_graph for more information.
 
-    See also
+    See Also
     --------
     dask.dot.dot_graph
 
@@ -248,22 +374,188 @@ def visualize(*args, **kwargs):
 
     http://dask.pydata.org/en/latest/optimize.html
     """
+    from dask.dot import dot_graph
 
-    dsks = [arg for arg in args if isinstance(arg, dict)]
-    args = [arg for arg in args if isinstance(arg, Base)]
     filename = kwargs.pop('filename', 'mydask')
     optimize_graph = kwargs.pop('optimize_graph', False)
-    from dask.dot import dot_graph
-    if optimize_graph:
-        dsks.extend([optimization_function(arg)(ensure_dict(arg.dask), arg._keys())
-                     for arg in args])
-    else:
-        dsks.extend([arg.dask for arg in args])
-    dsk = merge(dsks)
+
+    dsks = [arg for arg in args if isinstance(arg, dict)]
+    args = [arg for arg in args if is_dask_collection(arg)]
+
+    dsk = collections_to_dsk(args, optimize_graph=optimize_graph)
+    for d in dsks:
+        dsk.update(d)
 
     return dot_graph(dsk, filename=filename, **kwargs)
 
 
+def persist(*args, **kwargs):
+    """ Persist multiple Dask collections into memory
+
+    This turns lazy Dask collections into Dask collections with the same
+    metadata, but now with their results fully computed or actively computing
+    in the background.
+
+    For example a lazy dask.array built up from many lazy calls will now be a
+    dask.array of the same shape, dtype, chunks, etc., but now with all of
+    those previously lazy tasks either computed in memory as many small NumPy
+    arrays (in the single-machine case) or asynchronously running in the
+    background on a cluster (in the distributed case).
+
+    This function operates differently if a ``dask.distributed.Client`` exists
+    and is connected to a distributed scheduler.  In this case this function
+    will return as soon as the task graph has been submitted to the cluster,
+    but before the computations have completed.  Computations will continue
+    asynchronously in the background.  When using this function with the single
+    machine scheduler it blocks until the computations have finished.
+
+    When using Dask on a single machine you should ensure that the dataset fits
+    entirely within memory.
+
+    Examples
+    --------
+    >>> df = dd.read_csv('/path/to/*.csv')  # doctest: +SKIP
+    >>> df = df[df.name == 'Alice']  # doctest: +SKIP
+    >>> df['in-debt'] = df.balance < 0  # doctest: +SKIP
+    >>> df = df.persist()  # triggers computation  # doctest: +SKIP
+
+    >>> df.value().min()  # future computations are now fast  # doctest: +SKIP
+    -10
+    >>> df.value().max()  # doctest: +SKIP
+    100
+
+    >>> from dask import persist  # use persist function on multiple collections
+    >>> a, b = persist(a, b)  # doctest: +SKIP
+
+    Parameters
+    ----------
+    *args: Dask collections
+    get : callable, optional
+        A scheduler ``get`` function to use. If not provided, the default
+        is to check the global settings first, and then fall back to
+        the collection defaults.
+    optimize_graph : bool, optional
+        If True [default], the graph is optimized before computation.
+        Otherwise the graph is run as is. This can be useful for debugging.
+    **kwargs
+        Extra keywords to forward to the scheduler ``get`` function.
+
+    Returns
+    -------
+    New dask collections backed by in-memory data
+    """
+    collections = [a for a in args if is_dask_collection(a)]
+    if not collections:
+        return args
+
+    get = kwargs.pop('get', None) or _globals['get']
+
+    if get is None and getattr(thread_state, 'key', False):
+        from distributed.worker import get_worker
+        get = get_worker().client.get
+
+    if inspect.ismethod(get):
+        try:
+            from distributed.client import default_client
+        except ImportError:
+            pass
+        else:
+            try:
+                client = default_client()
+            except ValueError:
+                pass
+            else:
+                if client.get == _globals['get']:
+                    collections = client.persist(collections, **kwargs)
+                    if isinstance(collections, list):  # distributed is inconsistent here
+                        collections = tuple(collections)
+                    else:
+                        collections = (collections,)
+                    results_iter = iter(collections)
+                    return tuple(a if not is_dask_collection(a)
+                                 else next(results_iter)
+                                 for a in args)
+
+    optimize_graph = kwargs.pop('optimize_graph', True)
+
+    if not get:
+        get = collections[0].__dask_scheduler__
+        if not all(a.__dask_scheduler__ == get for a in collections):
+            raise ValueError("Compute called on multiple collections with "
+                             "differing default schedulers. Please specify a "
+                             "scheduler `get` function using either "
+                             "the `get` kwarg or globally with `set_options`.")
+
+    dsk = collections_to_dsk(collections, optimize_graph, **kwargs)
+
+    keys, postpersists = [], []
+    for a in args:
+        if is_dask_collection(a):
+            a_keys = list(flatten(a.__dask_keys__()))
+            rebuild, state = a.__dask_postpersist__()
+            keys.extend(a_keys)
+            postpersists.append((rebuild, a_keys, state))
+        else:
+            postpersists.append((None, None, a))
+
+    results = get(dsk, keys, **kwargs)
+    d = dict(zip(keys, results))
+    return tuple(s if r is None else r({k: d[k] for k in ks}, *s)
+                 for r, ks, s in postpersists)
+
+
+############
+# Tokenize #
+############
+
+def tokenize(*args, **kwargs):
+    """ Deterministic token
+
+    >>> tokenize([1, 2, '3'])
+    '7d6a880cd9ec03506eee6973ff551339'
+
+    >>> tokenize('Hello') == tokenize('Hello')
+    True
+    """
+    if kwargs:
+        args = args + (kwargs,)
+    return md5(str(tuple(map(normalize_token, args))).encode()).hexdigest()
+
+
+normalize_token = Dispatch()
+normalize_token.register((int, long, float, str, unicode, bytes, type(None),
+                          type, slice, complex, type(Ellipsis)),
+                         identity)
+
+
+@normalize_token.register(dict)
+def normalize_dict(d):
+    return normalize_token(sorted(d.items(), key=str))
+
+
+@normalize_token.register(OrderedDict)
+def normalize_ordered_dict(d):
+    return type(d).__name__, normalize_token(list(d.items()))
+
+
+@normalize_token.register(set)
+def normalize_set(s):
+    return normalize_token(sorted(s, key=str))
+
+
+@normalize_token.register((tuple, list))
+def normalize_seq(seq):
+    return type(seq).__name__, list(map(normalize_token, seq))
+
+
+@normalize_token.register(object)
+def normalize_object(o):
+    method = getattr(o, '__dask_tokenize__', None)
+    if method is not None:
+        return method()
+    return normalize_function(o) if callable(o) else uuid.uuid4().hex
+
+
 function_cache = {}
 function_cache_lock = threading.Lock()
 
@@ -299,54 +591,15 @@ def _normalize_function(func):
             result = pickle.dumps(func, protocol=0)
             if b'__main__' not in result:  # abort on dynamic functions
                 return result
-        except:
+        except Exception:
             pass
         try:
             import cloudpickle
             return cloudpickle.dumps(func, protocol=0)
-        except:
+        except Exception:
             return str(func)
 
 
-normalize_token = Dispatch()
-normalize_token.register((int, long, float, str, unicode, bytes, type(None),
-                          type, slice, complex),
-                         identity)
-
-
-@normalize_token.register(dict)
-def normalize_dict(d):
-    return normalize_token(sorted(d.items(), key=str))
-
-
-@normalize_token.register(OrderedDict)
-def normalize_ordered_dict(d):
-    return type(d).__name__, normalize_token(list(d.items()))
-
-
-@normalize_token.register(set)
-def normalize_set(s):
-    return normalize_token(sorted(s, key=str))
-
-
-@normalize_token.register((tuple, list))
-def normalize_seq(seq):
-    return type(seq).__name__, list(map(normalize_token, seq))
-
-
-@normalize_token.register(object)
-def normalize_object(o):
-    if callable(o):
-        return normalize_function(o)
-    else:
-        return uuid.uuid4().hex
-
-
-@normalize_token.register(Base)
-def normalize_base(b):
-    return type(b).__name__, b.key
-
-
 @normalize_token.register_lazy("pandas")
 def register_pandas():
     import pandas as pd
@@ -412,193 +665,5 @@ def register_numpy():
             name = x.__name__
             if getattr(np, name) is x:
                 return 'np.' + name
-        except:
+        except AttributeError:
             return normalize_function(x)
-
-
-def tokenize(*args, **kwargs):
-    """ Deterministic token
-
-    >>> tokenize([1, 2, '3'])
-    '7d6a880cd9ec03506eee6973ff551339'
-
-    >>> tokenize('Hello') == tokenize('Hello')
-    True
-    """
-    if kwargs:
-        args = args + (kwargs,)
-    return md5(str(tuple(map(normalize_token, args))).encode()).hexdigest()
-
-
-def dont_optimize(dsk, keys):
-    return dsk
-
-
-def optimization_function(obj):
-    if isinstance(obj, type):
-        cls = obj
-    else:
-        cls = type(obj)
-    name = cls.__name__.lower() + '_optimize' # dask.set_options(array_optimize=foo)
-    if name in _globals:
-        return _globals[name] or dont_optimize
-    try:
-        return cls._optimize
-    except AttributeError:
-        return dont_optimize
-
-
-def collections_to_dsk(collections, optimize_graph=True, **kwargs):
-    """
-    Convert many collections into a single dask graph, after optimization
-    """
-    optimizations = (kwargs.pop('optimizations', None) or
-                     _globals.get('optimizations', []))
-    if optimize_graph:
-        groups = groupby(optimization_function, collections)
-        groups = {opt: _extract_graph_and_keys(val)
-                  for opt, val in groups.items()}
-        for opt in optimizations:
-            groups = {k: [opt(ensure_dict(dsk), keys), keys]
-                      for k, (dsk, keys) in groups.items()}
-        dsk = merge([opt(dsk, keys, **kwargs)
-                     for opt, (dsk, keys) in groups.items()])
-    else:
-        dsk = ensure_dict(sharedict.merge(*[c.dask for c in collections]))
-
-    return dsk
-
-
-def _extract_graph_and_keys(vals):
-    """Given a list of dask vals, return a single graph and a list of keys such
-    that ``get(dsk, keys)`` is equivalent to ``[v.compute() v in vals]``."""
-    dsk = {}
-    keys = []
-    for v in vals:
-        d = v.dask
-        if type(d) is ShareDict:
-            for dd in d.dicts.values():
-                dsk.update(dd)
-        else:
-            dsk.update(v.dask)
-        keys.append(v._keys())
-
-    return dsk, keys
-
-
-def redict_collection(c, dsk):
-    cc = copy.copy(c)
-    cc.dask = dsk
-    return cc
-
-
-def persist(*args, **kwargs):
-    """ Persist multiple Dask collections into memory
-
-    This turns lazy Dask collections into Dask collections with the same
-    metadata, but now with their results fully computed or actively computing
-    in the background.
-
-    For example a lazy dask.array built up from many lazy calls will now be a
-    dask.array of the same shape, dtype, chunks, etc., but now with all of
-    those previously lazy tasks either computed in memory as many small NumPy
-    arrays (in the single-machine case) or asynchronously running in the
-    background on a cluster (in the distributed case).
-
-    This function operates differently if a ``dask.distributed.Client`` exists
-    and is connected to a distributed scheduler.  In this case this function
-    will return as soon as the task graph has been submitted to the cluster,
-    but before the computations have completed.  Computations will continue
-    asynchronously in the background.  When using this function with the single
-    machine scheduler it blocks until the computations have finished.
-
-    When using Dask on a single machine you should ensure that the dataset fits
-    entirely within memory.
-
-    Examples
-    --------
-    >>> df = dd.read_csv('/path/to/*.csv')  # doctest: +SKIP
-    >>> df = df[df.name == 'Alice']  # doctest: +SKIP
-    >>> df['in-debt'] = df.balance < 0  # doctest: +SKIP
-    >>> df = df.persist()  # triggers computation  # doctest: +SKIP
-
-    >>> df.value().min()  # future computations are now fast  # doctest: +SKIP
-    -10
-    >>> df.value().max()  # doctest: +SKIP
-    100
-
-    >>> from dask import persist  # use persist function on multiple collections
-    >>> a, b = persist(a, b)  # doctest: +SKIP
-
-    Parameters
-    ----------
-    *args: Dask collections
-    get : callable, optional
-        A scheduler ``get`` function to use. If not provided, the default
-        is to check the global settings first, and then fall back to
-        the collection defaults.
-    optimize_graph : bool, optional
-        If True [default], the graph is optimized before computation.
-        Otherwise the graph is run as is. This can be useful for debugging.
-    **kwargs
-        Extra keywords to forward to the scheduler ``get`` function.
-
-    Returns
-    -------
-    New dask collections backed by in-memory data
-    """
-    collections = [a for a in args if isinstance(a, Base)]
-    if not collections:
-        return args
-
-    get = kwargs.pop('get', None) or _globals['get']
-
-    if inspect.ismethod(get):
-        try:
-            from distributed.client import default_client
-        except ImportError:
-            pass
-        else:
-            try:
-                client = default_client()
-            except ValueError:
-                pass
-            else:
-                if client.get == _globals['get']:
-                    collections = client.persist(collections, **kwargs)
-                    if isinstance(collections, list):  # distributed is inconsistent here
-                        collections = tuple(collections)
-                    else:
-                        collections = (collections,)
-                    results_iter = iter(collections)
-                    return tuple(a if not isinstance(a, Base)
-                                 else next(results_iter)
-                                 for a in args)
-
-    optimize_graph = kwargs.pop('optimize_graph', True)
-
-    if not get:
-        get = collections[0]._default_get
-        if not all(a._default_get == get for a in collections):
-            raise ValueError("Compute called on multiple collections with "
-                             "differing default schedulers. Please specify a "
-                             "scheduler `get` function using either "
-                             "the `get` kwarg or globally with `set_options`.")
-
-    dsk = collections_to_dsk(collections, optimize_graph, **kwargs)
-    keys = list(flatten([var._keys() for var in collections]))
-    results = get(dsk, keys, **kwargs)
-
-    d = dict(zip(keys, results))
-
-    result = [redict_collection(c, {k: d[k]
-                                    for k in flatten(c._keys())})
-              for c in collections]
-    results_iter = iter(result)
-    return tuple(a if not isinstance(a, Base)
-                 else next(results_iter)
-                 for a in args)
-
-
-if PY3:
-    Base.persist.__doc__ = persist.__doc__
