@@ -14,7 +14,8 @@ from dask.utils import filetexts
 from dask.bytes import compression
 from dask.bytes.local import LocalFileSystem
 from dask.bytes.core import (open_text_files, write_bytes, read_bytes,
-                             open_files, OpenFileCreator)
+                             open_files, OpenFileCreator, FileSystem,
+                             get_pyarrow_filesystem)
 
 compute = partial(compute, get=get)
 
@@ -28,6 +29,21 @@ files = {'.test.accounts.1.json': (b'{"amount": 100, "name": "Alice"}\n'
                                    b'{"amount": 800, "name": "Dennis"}\n')}
 
 
+try:
+    # used only in test_with_urls - may be more generally useful
+    import pathlib
+
+    def to_uri(path):
+        return pathlib.Path(os.path.abspath(path)).as_uri()
+
+except (ImportError, NameError):
+    import urlparse, urllib
+
+    def to_uri(path):
+        return urlparse.urljoin(
+            'file:', urllib.pathname2url(os.path.abspath(path)))
+
+
 def test_read_bytes():
     with filetexts(files, mode='b'):
         sample, values = read_bytes('.test.accounts.*')
@@ -95,10 +111,6 @@ def test_with_paths():
         # relative path doesn't work
         url = pathlib.Path('file://.test.accounts.*')
         read_bytes(url, blocksize=None)
-    pytest.importorskip("s3fs")
-    with pytest.raises(ValueError):
-        url = pathlib.Path('s3://bucket/test.accounts.*')
-        sample, values = read_bytes(url, blocksize=None)
 
 
 def test_read_bytes_block():
@@ -366,16 +378,15 @@ def test_abs_paths(tmpdir):
     assert fs.open('tmp', 'r').read() == 'hi'
 
 
-try:
-    # used only in test_with_urls - may be more generally useful
-    import pathlib
+class UnknownFileSystem(FileSystem):
+    pass
 
-    def to_uri(path):
-        return pathlib.Path(os.path.abspath(path)).as_uri()
 
-except (ImportError, NameError):
-    import urlparse, urllib
+def test_get_pyarrow_filesystem():
+    pa = pytest.importorskip('pyarrow')
 
-    def to_uri(path):
-        return urlparse.urljoin(
-            'file:', urllib.pathname2url(os.path.abspath(path)))
+    fs = LocalFileSystem()
+    assert isinstance(get_pyarrow_filesystem(fs), pa.filesystem.LocalFileSystem)
+
+    with pytest.raises(NotImplementedError):
+        get_pyarrow_filesystem(UnknownFileSystem())
