@@ -1,3 +1,4 @@
+# flake8: noqa
 from __future__ import absolute_import, division, print_function
 
 import functools
@@ -36,8 +37,7 @@ if PY3:
         pass
 
     from urllib.request import urlopen
-    from urllib.parse import urlparse
-    from urllib.parse import quote, unquote
+    from urllib.parse import urlparse, urlsplit, quote, unquote
     FileNotFoundError = FileNotFoundError
     unicode = str
     long = int
@@ -54,6 +54,11 @@ if PY3:
     def _getargspec(func):
         return inspect.getfullargspec(func)
 
+    def reraise(exc, tb=None):
+        if exc.__traceback__ is not tb:
+            raise exc.with_traceback(tb)
+        raise exc
+
 else:
     import __builtin__ as builtins
     from Queue import Queue, Empty
@@ -63,7 +68,7 @@ else:
     import bz2
     import gzip
     from urllib2 import urlopen
-    from urlparse import urlparse
+    from urlparse import urlparse, urlsplit
     from urllib import quote, unquote
     unicode = unicode
     long = long
@@ -73,6 +78,16 @@ else:
     operator_div = operator.div
     FileNotFoundError = IOError
 
+    def _make_reraise():
+        _code = ("def reraise(exc, tb=None):"
+                "    raise type(exc), exc, tb")
+        namespace = {}
+        exec("exec _code in namespace")
+        return namespace['reraise']
+
+    reraise = _make_reraise()
+    del _make_reraise
+
     def _getargspec(func):
         return inspect.getargspec(func)
 
@@ -148,63 +163,60 @@ else:
     else:
         BZ2File = bz2.BZ2File
 
-    if sys.version_info[1] <= 6:
-        class GzipFile(BufferedIOBase):
-            def __init__(self, *args, **kwargs):
-                self.__obj = gzip.GzipFile(*args, **kwargs)
+    class GzipFile(BufferedIOBase):
+        def __init__(self, *args, **kwargs):
+            self.__obj = gzip.GzipFile(*args, **kwargs)
 
-            def close(self):
-                return self.__obj.close()
+        def close(self):
+            return self.__obj.close()
 
-            @property
-            def closed(self):
-                return self.__obj.fileobj is None
+        @property
+        def closed(self):
+            return self.__obj.fileobj is None
 
-            def flush(self, *args, **kwargs):
-                return self.__obj.flush(*args, **kwargs)
+        def flush(self, *args, **kwargs):
+            return self.__obj.flush(*args, **kwargs)
 
-            def isatty(self):
-                return self.__obj.isatty()
+        def isatty(self):
+            return self.__obj.isatty()
 
-            def read(self, *args, **kwargs):
-                return self.__obj.read(*args, **kwargs)
+        def read(self, *args, **kwargs):
+            return self.__obj.read(*args, **kwargs)
 
-            def read1(self, *args, **kwargs):
-                return self.__obj.read(*args, **kwargs)
+        def read1(self, *args, **kwargs):
+            return self.__obj.read(*args, **kwargs)
 
-            def readable(self):
-                return self.__obj.mode == gzip.READ
+        def readable(self):
+            return self.__obj.mode == gzip.READ
 
-            def readline(self, *args, **kwargs):
-                return self.__obj.readline(*args, **kwargs)
+        def readline(self, *args, **kwargs):
+            return self.__obj.readline(*args, **kwargs)
 
-            def readlines(self, *args, **kwargs):
-                return self.__obj.readlines(*args, **kwargs)
+        def readlines(self, *args, **kwargs):
+            return self.__obj.readlines(*args, **kwargs)
 
-            def seek(self, *args, **kwargs):
-                self.__obj.seek(*args, **kwargs)
-                return self.tell()
+        def seek(self, *args, **kwargs):
+            self.__obj.seek(*args, **kwargs)
+            return self.tell()
 
-            def seekable(self):
-                # See https://hg.python.org/cpython/file/2.7/Lib/gzip.py#l421
-                return True
+        def seekable(self):
+            # See https://hg.python.org/cpython/file/2.7/Lib/gzip.py#l421
+            return True
 
-            def tell(self):
-                return self.__obj.tell()
+        def tell(self):
+            return self.__obj.tell()
 
-            def truncate(self, *args, **kwargs):
-                return self.__obj.truncate(*args, **kwargs)
+        def truncate(self, *args, **kwargs):
+            return self.__obj.truncate(*args, **kwargs)
 
-            def writable(self):
-                return self.__obj.mode == gzip.WRITE
+        def writable(self):
+            return self.__obj.mode == gzip.WRITE
 
-            def write(self, *args, **kwargs):
-                return self.__obj.write(*args, **kwargs)
+        def write(self, *args, **kwargs):
+            return self.__obj.write(*args, **kwargs)
 
-            def writelines(self, *args, **kwargs):
-                return self.__obj.writelines(*args, **kwargs)
-    else:
-        GzipFile = gzip.GzipFile
+        def writelines(self, *args, **kwargs):
+            return self.__obj.writelines(*args, **kwargs)
 
     try:
         try:
@@ -230,9 +242,6 @@ def getargspec(func):
         else:
             return _getargspec(func)
 
-def skip(func):
-    return
-
 
 def bind_method(cls, name, func):
     """Bind a method to class
