@@ -1,7 +1,8 @@
 from __future__ import print_function, division, absolute_import
 
 from .core import (DataFrame, Series, Index, _Frame, map_partitions,
-                   repartition, to_delayed, to_datetime)
+                   repartition, to_delayed, to_datetime, to_timedelta)
+from .groupby import Aggregation
 from .io import (from_array, from_pandas, from_bcolz,
                  from_dask_array, read_hdf, read_sql_table,
                  from_delayed, read_csv, to_csv, read_table,
