@@ -16,15 +16,18 @@ from ...bytes.core import write_bytes
 from ...bytes.compression import seekable_files, files as cfiles
 from ...compatibility import PY2, PY3
 from ...delayed import delayed
+from ...utils import asciitable
 
 from ..utils import clear_known_categories, PANDAS_VERSION
 
 from .io import from_delayed
 
 if PANDAS_VERSION >= '0.20.0':
-    from pandas.api.types import is_integer_dtype, is_float_dtype
+    from pandas.api.types import (is_integer_dtype, is_float_dtype,
+                                  is_object_dtype, is_datetime64_any_dtype)
 else:
-    from pandas.types.common import is_integer_dtype, is_float_dtype
+    from pandas.types.common import (is_integer_dtype, is_float_dtype,
+                                     is_object_dtype, is_datetime64_any_dtype)
 
 
 delayed = delayed(pure=True)
@@ -77,30 +80,78 @@ def coerce_dtypes(df, dtypes):
     df: Pandas DataFrame
     dtypes: dict like {'x': float}
     """
+    bad_dtypes = []
+    bad_dates = []
+    errors = []
     for c in df.columns:
         if c in dtypes and df.dtypes[c] != dtypes[c]:
-            if is_float_dtype(df.dtypes[c]) and is_integer_dtype(dtypes[c]):
-                # There is a mismatch between floating and integer columns.
-                # Determine all mismatched and error.
-                mismatched = sorted(c for c in df.columns if
-                                    is_float_dtype(df.dtypes[c]) and
-                                    is_integer_dtype(dtypes[c]))
+            actual = df.dtypes[c]
+            desired = dtypes[c]
+            if is_float_dtype(actual) and is_integer_dtype(desired):
+                bad_dtypes.append((c, actual, desired))
+            elif is_object_dtype(actual) and is_datetime64_any_dtype(desired):
+                # This can only occur when parse_dates is specified, but an
+                # invalid date is encountered. Pandas then silently falls back
+                # to object dtype. Since `object_array.astype(datetime)` will
+                # silently overflow, error here and report.
+                bad_dates.append(c)
+            else:
+                try:
+                    df[c] = df[c].astype(dtypes[c])
+                except Exception as e:
+                    bad_dtypes.append((c, actual, desired))
+                    errors.append((c, e))
 
-                msg = ("Mismatched dtypes found.\n"
-                       "Expected integers, but found floats for columns:\n"
-                       "%s\n\n"
-                       "To fix, specify dtypes manually by adding:\n\n"
-                       "%s\n\n"
-                       "to the call to `read_csv`/`read_table`.\n\n"
-                       "Alternatively, provide `assume_missing=True` to "
-                       "interpret all unspecified integer columns as floats.")
+    if bad_dtypes:
+        if errors:
+            ex = '\n'.join("- %s\n  %r" % (c, e) for c, e in
+                           sorted(errors, key=lambda x: str(x[0])))
+            exceptions = ("The following columns also raised exceptions on "
+                          "conversion:\n\n%s\n\n") % ex
+            extra = ""
+        else:
+            exceptions = ""
+            # All mismatches are int->float, also suggest `assume_missing=True`
+            extra = ("\n\nAlternatively, provide `assume_missing=True` "
+                     "to interpret\n"
+                     "all unspecified integer columns as floats.")
 
-                missing_list = '\n'.join('- %r' % c for c in mismatched)
-                dtype_list = ('%r: float' % c for c in mismatched)
-                missing_dict = 'dtype={%s}' % ',\n       '.join(dtype_list)
-                raise ValueError(msg % (missing_list, missing_dict))
+        bad_dtypes = sorted(bad_dtypes, key=lambda x: str(x[0]))
+        table = asciitable(['Column', 'Found', 'Expected'], bad_dtypes)
+        dtype_kw = ('dtype={%s}' % ',\n'
+                    '       '.join("%r: '%s'" % (k, v)
+                                   for (k, v, _) in bad_dtypes))
 
-            df[c] = df[c].astype(dtypes[c])
+        dtype_msg = (
+            "{table}\n\n"
+            "{exceptions}"
+            "Usually this is due to dask's dtype inference failing, and\n"
+            "*may* be fixed by specifying dtypes manually by adding:\n\n"
+            "{dtype_kw}\n\n"
+            "to the call to `read_csv`/`read_table`."
+            "{extra}").format(table=table, exceptions=exceptions,
+                              dtype_kw=dtype_kw, extra=extra)
+    else:
+        dtype_msg = None
+
+    if bad_dates:
+        also = " also " if bad_dtypes else " "
+        cols = '\n'.join("- %s" % c for c in bad_dates)
+        date_msg = (
+            "The following columns{also}failed to properly parse as dates:\n\n"
+            "{cols}\n\n"
+            "This is usually due to an invalid value in that column. To\n"
+            "diagnose and fix it's recommended to drop these columns from the\n"
+            "`parse_dates` keyword, and manually convert them to dates later\n"
+            "using `dd.to_datetime`.").format(also=also, cols=cols)
+    else:
+        date_msg = None
+
+    if bad_dtypes or bad_dates:
+        rule = "\n\n%s\n\n" % ('-' * 61)
+        msg = ("Mismatched dtypes found in `pd.read_csv`/`pd.read_table`.\n\n"
+               "%s" % (rule.join(filter(None, [dtype_msg, date_msg]))))
+        raise ValueError(msg)
 
 
 def text_blocks_to_pandas(reader, block_lists, header, head, kwargs,
@@ -474,13 +525,18 @@ def to_csv(df, filename, name_function=None, compression=None, compute=True,
         European data
     storage_options: dict
         Parameters passed on to the backend filesystem class.
+    Returns
+    -------
+    The names of the file written if they were computed right away
+    If not, the delayed tasks associated to the writing of the files
     """
     values = [_to_csv_chunk(d, **kwargs) for d in df.to_delayed()]
-    values = write_bytes(values, filename, name_function, compression,
-                         encoding=None, **(storage_options or {}))
+    (values, names) = write_bytes(values, filename, name_function, compression,
+                                  encoding=None, **(storage_options or {}))
 
     if compute:
         delayed(values).compute(get=get)
+        return names
     else:
         return values
 
