@@ -1,31 +1,24 @@
 from __future__ import absolute_import, division, print_function
 
 from fnmatch import fnmatch
-from functools import wraps
 from glob import glob
 import os
-from threading import Lock
-import multiprocessing
 import uuid
 from warnings import warn
 
 import pandas as pd
-import dask
 from toolz import merge
 
-from ...async import get_sync
-from ...base import tokenize
+from .io import _link
+from ..core import DataFrame, new_dd_object
+from ... import multiprocessing
+from ...base import tokenize, compute_as_if_collection
+from ...bytes.utils import build_name_function
+from ...compatibility import PY3
 from ...context import _globals
 from ...delayed import Delayed, delayed
-import dask.multiprocessing
-
-from ..core import DataFrame, new_dd_object
-
-from ...utils import build_name_function
-
-from .io import _link
-
-lock = Lock()
+from ...local import get_sync
+from ...utils import effective_get, get_scheduler_lock
 
 
 def _pd_to_hdf(pd_to_hdf, lock, args, kwargs=None):
@@ -42,10 +35,87 @@ def _pd_to_hdf(pd_to_hdf, lock, args, kwargs=None):
     return None
 
 
-@wraps(pd.DataFrame.to_hdf)
-def to_hdf(df, path_or_buf, key, mode='a', append=False, get=None,
+def to_hdf(df, path, key, mode='a', append=False, get=None,
            name_function=None, compute=True, lock=None, dask_kwargs={},
            **kwargs):
+    """ Store Dask Dataframe to Hierarchical Data Format (HDF) files
+
+    This is a parallel version of the Pandas function of the same name.  Please
+    see the Pandas docstring for more detailed information about shared keyword
+    arguments.
+
+    This function differs from the Pandas version by saving the many partitions
+    of a Dask DataFrame in parallel, either to many files, or to many datasets
+    within the same file.  You may specify this parallelism with an asterix
+    ``*`` within the filename or datapath, and an optional ``name_function``.
+    The asterix will be replaced with an increasing sequence of integers
+    starting from ``0`` or with the result of calling ``name_function`` on each
+    of those integers.
+
+    This function only supports the Pandas ``'table'`` format, not the more
+    specialized ``'fixed'`` format.
+
+    Parameters
+    ----------
+    path: string
+        Path to a target filename.  May contain a ``*`` to denote many filenames
+    key: string
+        Datapath within the files.  May contain a ``*`` to denote many locations
+    name_function: function
+        A function to convert the ``*`` in the above options to a string.
+        Should take in a number from 0 to the number of partitions and return a
+        string. (see examples below)
+    compute: bool
+        Whether or not to execute immediately.  If False then this returns a
+        ``dask.Delayed`` value.
+    lock: Lock, optional
+        Lock to use to prevent concurrency issues.  By default a
+        ``threading.Lock``, ``multiprocessing.Lock`` or ``SerializableLock``
+        will be used depending on your scheduler if a lock is required. See
+        dask.utils.get_scheduler_lock for more information about lock
+        selection.
+    **other:
+        See pandas.to_hdf for more information
+
+    Examples
+    --------
+    Save Data to a single file
+
+    >>> df.to_hdf('output.hdf', '/data')            # doctest: +SKIP
+
+    Save data to multiple datapaths within the same file:
+
+    >>> df.to_hdf('output.hdf', '/data-*')          # doctest: +SKIP
+
+    Save data to multiple files:
+
+    >>> df.to_hdf('output-*.hdf', '/data')          # doctest: +SKIP
+
+    Save data to multiple files, using the multiprocessing scheduler:
+
+    >>> df.to_hdf('output-*.hdf', '/data', get=dask.multiprocessing.get) # doctest: +SKIP
+
+    Specify custom naming scheme.  This writes files as
+    '2000-01-01.hdf', '2000-01-02.hdf', '2000-01-03.hdf', etc..
+
+    >>> from datetime import date, timedelta
+    >>> base = date(year=2000, month=1, day=1)
+    >>> def name_function(i):
+    ...     ''' Convert integer 0 to n to a string '''
+    ...     return base + timedelta(days=i)
+
+    >>> df.to_hdf('*.hdf', '/data', name_function=name_function) # doctest: +SKIP
+
+    Returns
+    -------
+    None: if compute == True
+    delayed value: if compute == False
+
+    See Also
+    --------
+    read_hdf:
+    to_parquet:
+    """
     name = 'to-hdf-' + uuid.uuid1().hex
 
     pd_to_hdf = getattr(df._partition_type, 'to_hdf')
@@ -53,20 +123,22 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, get=None,
     single_file = True
     single_node = True
 
-    # if path_or_buf is string, format using i_name
-    if isinstance(path_or_buf, str):
-        if path_or_buf.count('*') + key.count('*') > 1:
-            raise ValueError("A maximum of one asterisk is accepted in file path and dataset key")
+    # if path is string, format using i_name
+    if isinstance(path, str):
+        if path.count('*') + key.count('*') > 1:
+            raise ValueError("A maximum of one asterisk is accepted in file "
+                             "path and dataset key")
 
-        fmt_obj = lambda path_or_buf, i_name: path_or_buf.replace('*', i_name)
+        fmt_obj = lambda path, i_name: path.replace('*', i_name)
 
-        if '*' in path_or_buf:
+        if '*' in path:
             single_file = False
     else:
         if key.count('*') > 1:
-            raise ValueError("A maximum of one asterisk is accepted in dataset key")
+            raise ValueError("A maximum of one asterisk is accepted in "
+                             "dataset key")
 
-        fmt_obj = lambda path_or_buf, _: path_or_buf
+        fmt_obj = lambda path, _: path
 
     if '*' in key:
         single_node = False
@@ -94,22 +166,18 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, get=None,
         get = get_sync
 
     # handle lock default based on whether we're writing to a single entity
-    _actual_get = get or _globals.get('get') or df._default_get
+    _actual_get = effective_get(get, df)
     if lock is None:
         if not single_node:
             lock = True
-        elif not single_file and _actual_get is not dask.multiprocessing.get:
+        elif not single_file and _actual_get is not multiprocessing.get:
             # if we're writing to multiple files with the multiprocessing
             # scheduler we don't need to lock
             lock = True
         else:
             lock = False
-
-    if lock is True:
-        if _actual_get == dask.multiprocessing.get:
-            lock = multiprocessing.Manager().Lock()
-        else:
-            lock = Lock()
+    if lock:
+        lock = get_scheduler_lock(get, df)
 
     kwargs.update({'format': 'table', 'mode': mode, 'append': append})
 
@@ -117,7 +185,7 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, get=None,
 
     i_name = name_function(0)
     dsk[(name, 0)] = (_pd_to_hdf, pd_to_hdf, lock,
-                      [(df._name, 0), fmt_obj(path_or_buf, i_name),
+                      [(df._name, 0), fmt_obj(path, i_name),
                        key.replace('*', i_name)], kwargs)
 
     kwargs2 = kwargs.copy()
@@ -126,10 +194,15 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, get=None,
     if single_node:
         kwargs2['append'] = True
 
+    filenames = []
+    for i in range(0,df.npartitions):
+        i_name = name_function(i)
+        filenames.append(fmt_obj(path, i_name))
+
     for i in range(1, df.npartitions):
         i_name = name_function(i)
         task = (_pd_to_hdf, pd_to_hdf, lock,
-                [(df._name, i), fmt_obj(path_or_buf, i_name),
+                [(df._name, i), fmt_obj(path, i_name),
                     key.replace('*', i_name)], kwargs2)
         if single_file:
             link_dep = i - 1 if single_node else 0
@@ -143,9 +216,10 @@ def to_hdf(df, path_or_buf, key, mode='a', append=False, get=None,
         keys = [(name, i) for i in range(df.npartitions)]
 
     if compute:
-        return DataFrame._get(dsk, keys, get=get, **dask_kwargs)
+        compute_as_if_collection(DataFrame, dsk, keys, get=get, **dask_kwargs)
+        return filenames
     else:
-        return delayed([Delayed(k, [dsk]) for k in keys])
+        return delayed([Delayed(k, dsk) for k in keys])
 
 
 dont_use_fixed_error_message = """
@@ -260,25 +334,42 @@ def _pd_read_hdf(path, key, lock, kwargs):
     return result
 
 
-@wraps(pd.read_hdf)
 def read_hdf(pattern, key, start=0, stop=None, columns=None,
              chunksize=1000000, sorted_index=False, lock=True, mode='a'):
     """
-    Read hdf files into a dask dataframe. Like pandas.read_hdf, except it we
-    can read multiple files, and read multiple keys from the same file by using
-    pattern matching.
+    Read HDF files into a Dask DataFrame
+
+    Read hdf files into a dask dataframe. This function is like
+    ``pandas.read_hdf``, except it can read from a single large file, or from
+    multiple files, or from multiple keys from the same file.
 
     Parameters
     ----------
-    pattern : pattern (string), or buffer to read from. Can contain wildcards
+    pattern : string, list
+        File pattern (string), buffer to read from, or list of file
+        paths. Can contain wildcards.
     key : group identifier in the store. Can contain wildcards
     start : optional, integer (defaults to 0), row number to start at
     stop : optional, integer (defaults to None, the last row), row number to
         stop at
-    columns : optional, a list of columns that if not None, will limit the
-        return columns
-    chunksize : optional, positive integer
-        maximal number of rows per partition
+    columns : list of columns, optional
+        A list of columns that if not None, will limit the return
+        columns (default is None)
+    chunksize : positive integer, optional
+        Maximal number of rows per partition (default is 1000000).
+    sorted_index : boolean, optional
+        Option to specify whether or not the input hdf files have a sorted
+        index (default is False).
+    lock : boolean, optional
+        Option to use a lock to prevent concurrency issues (default is True).
+    mode : {'a', 'r', 'r+'}, default 'a'. Mode to use when opening file(s).
+        'r'
+            Read-only; no data can be modified.
+        'a'
+            Append; an existing file is opened for reading and writing,
+            and if the file does not exist it is created.
+        'r+'
+            It is similar to 'a', but the file must already exist.
 
     Returns
     -------
@@ -294,15 +385,20 @@ def read_hdf(pattern, key, start=0, stop=None, columns=None,
 
     >>> dd.read_hdf('myfile.*.hdf5', '/x')  # doctest: +SKIP
 
+    >>> dd.read_hdf(['myfile.1.hdf5', 'myfile.2.hdf5'], '/x')  # doctest: +SKIP
+
     Load multiple datasets
 
     >>> dd.read_hdf('myfile.1.hdf5', '/*')  # doctest: +SKIP
     """
     if lock is True:
-        lock = Lock()
+        lock = get_scheduler_lock()
 
     key = key if key.startswith('/') else '/' + key
-    paths = sorted(glob(pattern))
+    if isinstance(pattern, str):
+        paths = sorted(glob(pattern))
+    else:
+        paths = pattern
     if (start != 0 or stop is not None) and len(paths) > 1:
         raise NotImplementedError(read_hdf_error_msg)
     if chunksize <= 0:
@@ -316,3 +412,8 @@ def read_hdf(pattern, key, start=0, stop=None, columns=None,
                                     sorted_index=sorted_index,
                                     lock=lock, mode=mode)
                    for path in paths])
+
+
+if PY3:
+    from ..core import _Frame
+    _Frame.to_hdf.__doc__ = to_hdf.__doc__
