@@ -16,7 +16,7 @@ from ...delayed import delayed
 
 from ..core import DataFrame, Series, new_dd_object
 from ..shuffle import set_partition
-from ..utils import insert_meta_param_description
+from ..utils import insert_meta_param_description, check_meta, make_meta
 
 from ...utils import M, ensure_dict
 
@@ -240,7 +240,7 @@ def from_bcolz(x, chunksize=None, categorize=True, index=None, lock=lock,
                     np.issubdtype(x.dtype[name], np.unicode_) or
                     np.issubdtype(x.dtype[name], np.object_)):
                 a = da.from_array(x[name], chunks=(chunksize * len(x.names),))
-                categories[name] = da.unique(a)
+                categories[name] = da.unique(a).compute()
 
     columns = tuple(x.dtype.names)
     divisions = tuple(range(0, len(x), chunksize))
@@ -390,7 +390,7 @@ def from_dask_array(x, columns=None):
         divisions[-1] -= 1
 
     dsk = {}
-    for i, (chunk, ind) in enumerate(zip(x._keys(), index)):
+    for i, (chunk, ind) in enumerate(zip(x.__dask_keys__(), index)):
         if x.ndim == 2:
             chunk = chunk[0]
         if isinstance(meta, pd.Series):
@@ -435,8 +435,8 @@ def to_bag(df, index=False):
         raise TypeError("df must be either DataFrame or Series")
     name = 'to_bag-' + tokenize(df, index)
     dsk = dict(((name, i), (_df_to_bag, block, index))
-               for (i, block) in enumerate(df._keys()))
-    dsk.update(df._optimize(df.dask, df._keys()))
+               for (i, block) in enumerate(df.__dask_keys__()))
+    dsk.update(df.__dask_optimize__(df.__dask_graph__(), df.__dask_keys__()))
     return Bag(dsk, name, df.npartitions)
 
 
@@ -462,7 +462,7 @@ def to_records(df):
         raise TypeError("df must be either DataFrame or Series")
     name = 'to-records-' + tokenize(df)
     dsk = {(name, i): (M.to_records, key)
-           for (i, key) in enumerate(df._keys())}
+           for (i, key) in enumerate(df.__dask_keys__())}
     x = df._meta.to_records()
     chunks = ((np.nan,) * df.npartitions,)
     return Array(merge(df.dask, dsk), name, chunks, x.dtype)
@@ -499,20 +499,15 @@ def from_delayed(dfs, meta=None, divisions=None, prefix='from-delayed'):
         if not isinstance(df, Delayed):
             raise TypeError("Expected Delayed object, got %s" %
                             type(df).__name__)
-    dsk = merge(df.dask for df in dfs)
-
-    name = prefix + '-' + tokenize(*dfs)
-    names = [(name, i) for i in range(len(dfs))]
-    values = [df.key for df in dfs]
-    dsk2 = dict(zip(names, values))
-    dsk3 = merge(dsk, dsk2)
 
     if meta is None:
         meta = dfs[0].compute()
-    if isinstance(meta, (str, pd.Series)):
-        Frame = Series
-    else:
-        Frame = DataFrame
+    meta = make_meta(meta)
+
+    name = prefix + '-' + tokenize(*dfs)
+    dsk = merge(df.dask for df in dfs)
+    dsk.update({(name, i): (check_meta, df.key, meta, 'from_delayed')
+                for (i, df) in enumerate(dfs)})
 
     if divisions is None or divisions == 'sorted':
         divs = [None] * (len(dfs) + 1)
@@ -521,7 +516,7 @@ def from_delayed(dfs, meta=None, divisions=None, prefix='from-delayed'):
         if len(divs) != len(dfs) + 1:
             raise ValueError("divisions should be a tuple of len(dfs) + 1")
 
-    df = Frame(dsk3, name, meta, divs)
+    df = new_dd_object(dsk, name, meta, divs)
 
     if divisions == 'sorted':
         from ..shuffle import compute_divisions
