@@ -15,6 +15,7 @@ import pandas.util.testing as tm
 
 import dask
 import dask.dataframe as dd
+from dask.base import compute_as_if_collection
 from dask.dataframe.io.csv import (text_blocks_to_pandas, pandas_read_text,
                                    auto_blocksize)
 from dask.dataframe.utils import assert_eq, has_known_categories, PANDAS_VERSION
@@ -279,7 +280,9 @@ def test_read_csv_index():
         result = f.compute(get=dask.get)
         assert result.index.name == 'amount'
 
-        blocks = dd.DataFrame._get(f.dask, f._keys(), get=dask.get)
+        blocks = compute_as_if_collection(dd.DataFrame, f.dask,
+                                          f.__dask_keys__(),
+                                          get=dask.get)
         for i, block in enumerate(blocks):
             if i < len(f.divisions) - 2:
                 assert (block.index < f.divisions[i + 1]).all()
@@ -561,35 +564,108 @@ def test_read_csv_of_modified_file_has_different_name():
 
 
 def test_late_dtypes():
-    text = 'numbers,names,more_numbers,integers\n'
+    text = 'numbers,names,more_numbers,integers,dates\n'
     for i in range(1000):
-        text += '1,foo,2,3\n'
-    text += '1.5,bar,2.5,3\n'
+        text += '1,,2,3,2017-10-31 00:00:00\n'
+    text += '1.5,bar,2.5,3,4998-01-01 00:00:00\n'
+
+    date_msg = ("\n"
+                "\n"
+                "-------------------------------------------------------------\n"
+                "\n"
+                "The following columns also failed to properly parse as dates:\n"
+                "\n"
+                "- dates\n"
+                "\n"
+                "This is usually due to an invalid value in that column. To\n"
+                "diagnose and fix it's recommended to drop these columns from the\n"
+                "`parse_dates` keyword, and manually convert them to dates later\n"
+                "using `dd.to_datetime`.")
+
     with filetext(text) as fn:
         sol = pd.read_csv(fn)
+        msg = ("Mismatched dtypes found in `pd.read_csv`/`pd.read_table`.\n"
+               "\n"
+               "+--------------+---------+----------+\n"
+               "| Column       | Found   | Expected |\n"
+               "+--------------+---------+----------+\n"
+               "| more_numbers | float64 | int64    |\n"
+               "| names        | object  | float64  |\n"
+               "| numbers      | float64 | int64    |\n"
+               "+--------------+---------+----------+\n"
+               "\n"
+               "- names\n"
+               "  ValueError(.*)\n"
+               "\n"
+               "Usually this is due to dask's dtype inference failing, and\n"
+               "*may* be fixed by specifying dtypes manually by adding:\n"
+               "\n"
+               "dtype={'more_numbers': 'float64',\n"
+               "       'names': 'object',\n"
+               "       'numbers': 'float64'}\n"
+               "\n"
+               "to the call to `read_csv`/`read_table`.")
+
+        with pytest.raises(ValueError) as e:
+            dd.read_csv(fn, sample=50,
+                        parse_dates=['dates']).compute(get=dask.get)
+        assert e.match(msg + date_msg)
+
         with pytest.raises(ValueError) as e:
             dd.read_csv(fn, sample=50).compute(get=dask.get)
+        assert e.match(msg)
 
-        msg = ("Mismatched dtypes found.\n"
-               "Expected integers, but found floats for columns:\n"
-               "- 'more_numbers'\n"
-               "- 'numbers'\n"
+        msg = ("Mismatched dtypes found in `pd.read_csv`/`pd.read_table`.\n"
                "\n"
-               "To fix, specify dtypes manually by adding:\n"
+               "+--------------+---------+----------+\n"
+               "| Column       | Found   | Expected |\n"
+               "+--------------+---------+----------+\n"
+               "| more_numbers | float64 | int64    |\n"
+               "| numbers      | float64 | int64    |\n"
+               "+--------------+---------+----------+\n"
                "\n"
-               "dtype={'more_numbers': float,\n"
-               "       'numbers': float}\n"
+               "Usually this is due to dask's dtype inference failing, and\n"
+               "*may* be fixed by specifying dtypes manually by adding:\n"
+               "\n"
+               "dtype={'more_numbers': 'float64',\n"
+               "       'numbers': 'float64'}\n"
                "\n"
                "to the call to `read_csv`/`read_table`.\n"
                "\n"
-               "Alternatively, provide `assume_missing=True` to interpret "
+               "Alternatively, provide `assume_missing=True` to interpret\n"
                "all unspecified integer columns as floats.")
 
+        with pytest.raises(ValueError) as e:
+            dd.read_csv(fn, sample=50,
+                        dtype={'names': 'O'}).compute(get=dask.get)
+        assert str(e.value) == msg
+
+        with pytest.raises(ValueError) as e:
+            dd.read_csv(fn, sample=50, parse_dates=['dates'],
+                        dtype={'names': 'O'}).compute(get=dask.get)
+        assert str(e.value) == msg + date_msg
+
+        msg = ("Mismatched dtypes found in `pd.read_csv`/`pd.read_table`.\n"
+               "\n"
+               "The following columns failed to properly parse as dates:\n"
+               "\n"
+               "- dates\n"
+               "\n"
+               "This is usually due to an invalid value in that column. To\n"
+               "diagnose and fix it's recommended to drop these columns from the\n"
+               "`parse_dates` keyword, and manually convert them to dates later\n"
+               "using `dd.to_datetime`.")
+
+        with pytest.raises(ValueError) as e:
+            dd.read_csv(fn, sample=50, parse_dates=['dates'],
+                        dtype={'more_numbers': float, 'names': object,
+                               'numbers': float}).compute(get=dask.get)
         assert str(e.value) == msg
 
         # Specifying dtypes works
         res = dd.read_csv(fn, sample=50,
-                          dtype={'more_numbers': float, 'numbers': float})
+                          dtype={'more_numbers': float, 'names': object,
+                                 'numbers': float})
         assert_eq(res, sol)
 
 
@@ -915,3 +991,11 @@ def test_to_csv_with_get():
         assert flag[0]
         result = dd.read_csv(os.path.join(dn, '*')).compute().reset_index(drop=True)
         assert_eq(result, df)
+
+
+def test_to_csv_paths():
+    df = pd.DataFrame({"A": range(10)})
+    ddf = dd.from_pandas(df, npartitions=2)
+    assert ddf.to_csv("foo*.csv") == ['foo0.csv', 'foo1.csv']
+    os.remove('foo0.csv')
+    os.remove('foo1.csv')
