@@ -12,7 +12,6 @@ from time import sleep
 import dask.dataframe as dd
 
 from dask.utils import tmpfile, tmpdir, dependency_depth
-from dask.async import get_sync
 
 from dask.dataframe.utils import assert_eq
 
@@ -280,7 +279,7 @@ def test_to_hdf_lock_delays():
 
     # saving to multiple hdf nodes
     with tmpfile() as fn:
-        a = a.apply(delayed_nop, axis=1, columns=a.columns)
+        a = a.apply(delayed_nop, axis=1, meta=a)
         a.to_hdf(fn, '/data*')
         out = dd.read_hdf(fn, '/data*')
         assert_eq(df16, out)
@@ -289,7 +288,7 @@ def test_to_hdf_lock_delays():
     # adding artifichial delays to make sure last tasks finish first
     with tmpdir() as dn:
         fn = os.path.join(dn, 'data*')
-        a = a.apply(delayed_nop, axis=1, columns=a.columns)
+        a = a.apply(delayed_nop, axis=1, meta=a)
         a.to_hdf(fn, '/data')
         out = dd.read_hdf(fn, '/data')
         assert_eq(df16, out)
@@ -314,83 +313,33 @@ def test_to_hdf_exceptions():
                 a.to_hdf(hdf, '/data_*_*')
 
 
-def test_to_hdf_sync():
+@pytest.mark.parametrize('get', [dask.get,
+                                 dask.threaded.get,
+                                 dask.multiprocessing.get])
+@pytest.mark.parametrize('npartitions', [1, 4, 10])
+def test_to_hdf_schedulers(get, npartitions):
     pytest.importorskip('tables')
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
                        'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
                       index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
-    a = dd.from_pandas(df, 16)
+    a = dd.from_pandas(df, npartitions=npartitions)
 
     # test single file single node
     with tmpfile('h5') as fn:
-        a.to_hdf(fn, '/data', get=get_sync)
+        a.to_hdf(fn, '/data', get=get)
         out = pd.read_hdf(fn, '/data')
         assert_eq(df, out)
 
     # test multiple files single node
     with tmpdir() as dn:
         fn = os.path.join(dn, 'data_*.h5')
-        a.to_hdf(fn, '/data', get=get_sync)
+        a.to_hdf(fn, '/data', get=get)
         out = dd.read_hdf(fn, '/data')
         assert_eq(df, out)
 
     # test single file multiple nodes
     with tmpfile('h5') as fn:
-        a.to_hdf(fn, '/data*', get=get_sync)
-        out = dd.read_hdf(fn, '/data*')
-        assert_eq(df, out)
-
-
-def test_to_hdf_thread():
-    pytest.importorskip('tables')
-    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
-                      index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
-    a = dd.from_pandas(df, 16)
-
-    # test single file single node
-    with tmpfile('h5') as fn:
-        a.to_hdf(fn, '/data', get=dask.threaded.get)
-        out = pd.read_hdf(fn, '/data')
-        assert_eq(df, out)
-
-    # test multiple files single node
-    with tmpdir() as dn:
-        fn = os.path.join(dn, 'data_*.h5')
-        a.to_hdf(fn, '/data', get=dask.threaded.get)
-        out = dd.read_hdf(fn, '/data')
-        assert_eq(df, out)
-
-    # test single file multiple nodes
-    with tmpfile('h5') as fn:
-        a.to_hdf(fn, '/data*', get=dask.threaded.get)
-        out = dd.read_hdf(fn, '/data*')
-        assert_eq(df, out)
-
-
-def test_to_hdf_process():
-    pytest.importorskip('tables')
-    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p'],
-                       'y': [1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16]},
-                      index=[1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., 15., 16.])
-    a = dd.from_pandas(df, 16)
-
-    # test single file single node
-    with tmpfile('h5') as fn:
-        a.to_hdf(fn, '/data', get=dask.multiprocessing.get)
-        out = pd.read_hdf(fn, '/data')
-        assert_eq(df, out)
-
-    # test multiple files single node
-    with tmpdir() as dn:
-        fn = os.path.join(dn, 'data_*.h5')
-        a.to_hdf(fn, '/data', get=dask.multiprocessing.get)
-        out = dd.read_hdf(fn, '/data')
-        assert_eq(df, out)
-
-    # test single file multiple nodes
-    with tmpfile('h5') as fn:
-        a.to_hdf(fn, '/data*', get=dask.multiprocessing.get)
+        a.to_hdf(fn, '/data*', get=get)
         out = dd.read_hdf(fn, '/data*')
         assert_eq(df, out)
 
@@ -546,6 +495,21 @@ def test_hdf_globbing():
             tm.assert_frame_equal(res.compute(), pd.concat([df] * 3))
 
 
+def test_hdf_file_list():
+    pytest.importorskip('tables')
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+
+    with tmpdir() as tdir:
+        df.iloc[:2].to_hdf(os.path.join(tdir, 'one.h5'), 'dataframe', format='table')
+        df.iloc[2:].to_hdf(os.path.join(tdir, 'two.h5'), 'dataframe', format='table')
+
+        with dask.set_options(get=dask.get):
+            input_files = [os.path.join(tdir, 'one.h5'), os.path.join(tdir, 'two.h5')]
+            res = dd.read_hdf(input_files, 'dataframe')
+            tm.assert_frame_equal(res.compute(), df)
+
+
 def test_read_hdf_doesnt_segfault():
     pytest.importorskip('tables')
     with tmpfile('h5') as fn:
@@ -556,3 +520,12 @@ def test_read_hdf_doesnt_segfault():
 
         ddf = dd.read_hdf(fn, '/x', chunksize=2)
         assert len(ddf) == N
+
+
+def test_hdf_filenames():
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'd'],
+                       'y': [1, 2, 3, 4]}, index=[1., 2., 3., 4.])
+    ddf = dd.from_pandas(df, npartitions=2)
+    assert ddf.to_hdf("foo*.hdf5", "key") == ["foo0.hdf5", "foo1.hdf5"]
+    os.remove("foo0.hdf5")
+    os.remove("foo1.hdf5")
