@@ -55,30 +55,21 @@ We proceed with hash joins in the following stages:
 """
 from __future__ import absolute_import, division, print_function
 
-from bisect import bisect_left, bisect_right
-from functools import wraps
+from functools import wraps, partial
 from warnings import warn
 
-from toolz import merge_sorted, unique, partial, first
+from toolz import merge_sorted, unique, first
 import toolz
-import numpy as np
 import pandas as pd
 
 from ..base import tokenize
 from ..compatibility import apply
-from .core import (_Frame, DataFrame, map_partitions,
-                   Index, _maybe_from_pandas, new_dd_object)
+from .core import (_Frame, DataFrame, Series, map_partitions, Index,
+                   _maybe_from_pandas, new_dd_object, is_broadcastable)
 from .io import from_pandas
+from . import methods
 from .shuffle import shuffle, rearrange_by_divisions
-
-
-def bound(seq, left, right):
-    """ Bound sorted list by left and right values
-
-    >>> bound([1, 3, 4, 5, 8, 10, 12], 4, 10)
-    [4, 5, 8, 10]
-    """
-    return seq[bisect_left(seq, left): bisect_right(seq, right)]
+from .utils import strip_unknown_categories
 
 
 def align_partitions(*dfs):
@@ -102,14 +93,20 @@ def align_partitions(*dfs):
         A list of lists of keys that show which data exist on which
         divisions
     """
-    dfs1 = [df for df in dfs if isinstance(df, _Frame)]
+    _is_broadcastable = partial(is_broadcastable, dfs)
+    dfs1 = [df for df in dfs
+            if isinstance(df, _Frame) and
+            not _is_broadcastable(df)]
     if len(dfs) == 0:
         raise ValueError("dfs contains no DataFrame and Series")
     if not all(df.known_divisions for df in dfs1):
         raise ValueError("Not all divisions are known, can't align "
-                         "partitions. Please use `set_index` or "
-                         "`set_partition` to set the index.")
+                         "partitions. Please use `set_index` "
+                         "to set the index.")
+
     divisions = list(unique(merge_sorted(*[df.divisions for df in dfs1])))
+    if len(divisions) == 1:  # single value for index
+        divisions = (divisions[0], divisions[0])
     dfs2 = [df.repartition(divisions, force=True)
             if isinstance(df, _Frame) else df for df in dfs]
 
@@ -138,7 +135,10 @@ def _maybe_align_partitions(args):
     Note that if all divisions are unknown, but have equal npartitions, then
     they will be passed through unchanged. This is different than
     `align_partitions`, which will fail if divisions aren't all known"""
-    dfs = [df for df in args if isinstance(df, _Frame)]
+    _is_broadcastable = partial(is_broadcastable, args)
+    dfs = [df for df in args
+           if isinstance(df, _Frame) and
+           not _is_broadcastable(df)]
     if not dfs:
         return args
 
@@ -198,6 +198,7 @@ def require(divisions, parts, required=None):
 # Join / Merge
 ###############################################################
 
+
 required = {'left': [0], 'right': [1], 'inner': [0, 1], 'outer': []}
 
 
@@ -221,30 +222,15 @@ def merge_indexed_dataframes(lhs, rhs, how='left', lsuffix='', rsuffix='',
         if b is None and how in ('left', 'outer'):
             b = right_empty
 
-        dsk[(name, i)] = (_pdmerge, a, b, how, None, None, True, True,
+        dsk[(name, i)] = (methods.merge, a, b, how, None, None, True, True,
                           indicator, (lsuffix, rsuffix), left_empty,
                           right_empty)
 
     meta = pd.merge(lhs._meta_nonempty, rhs._meta_nonempty, how=how,
                     left_index=True, right_index=True,
                     suffixes=(lsuffix, rsuffix), indicator=indicator)
-    return DataFrame(toolz.merge(lhs.dask, rhs.dask, dsk),
-                     name, meta, divisions)
-
-
-def _pdmerge(left, right, how, left_on, right_on,
-             left_index, right_index, indicator, suffixes,
-             default_left, default_right):
-
-    if not len(left):
-        left = default_left
-
-    if not len(right):
-        right = default_right
-
-    return pd.merge(left, right, how=how, left_on=left_on, right_on=right_on,
-                    left_index=left_index, right_index=right_index,
-                    suffixes=suffixes, indicator=indicator)
+    return new_dd_object(toolz.merge(lhs.dask, rhs.dask, dsk),
+                         name, meta, divisions)
 
 
 shuffle_func = shuffle  # name sometimes conflicts with keyword argument
@@ -284,10 +270,6 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
                     left_index=left_index, right_index=right_index,
                     suffixes=suffixes, indicator=indicator)
 
-    merger = partial(_pdmerge, suffixes=suffixes,
-                     default_left=lhs._meta,
-                     default_right=rhs._meta)
-
     if isinstance(left_on, list):
         left_on = (list, tuple(left_on))
     if isinstance(right_on, list):
@@ -297,111 +279,45 @@ def hash_join(lhs, left_on, rhs, right_on, how='inner',
                      how, npartitions, suffixes, shuffle, indicator)
     name = 'hash-join-' + token
 
-    dsk = dict(((name, i), (merger, (lhs2._name, i), (rhs2._name, i),
+    dsk = dict(((name, i), (methods.merge, (lhs2._name, i), (rhs2._name, i),
                             how, left_on, right_on,
-                            left_index, right_index, indicator))
-                for i in range(npartitions))
+                            left_index, right_index, indicator,
+                            suffixes, lhs._meta, rhs._meta))
+               for i in range(npartitions))
 
     divisions = [None] * (npartitions + 1)
-    return DataFrame(toolz.merge(lhs2.dask, rhs2.dask, dsk),
-                     name, meta, divisions)
+    return new_dd_object(toolz.merge(lhs2.dask, rhs2.dask, dsk),
+                         name, meta, divisions)
 
 
 def single_partition_join(left, right, **kwargs):
+    # if the merge is perfomed on_index, divisions can be kept, otherwise the
+    # new index will not necessarily correspond the current divisions
+
     meta = pd.merge(left._meta_nonempty, right._meta_nonempty, **kwargs)
     name = 'merge-' + tokenize(left, right, **kwargs)
     if left.npartitions == 1:
-        left_key = first(left._keys())
-        dsk = dict(((name, i), (apply, pd.merge, [left_key, right_key],
-                                kwargs))
-                   for i, right_key in enumerate(right._keys()))
-        divisions = right.divisions
-    elif right.npartitions == 1:
-        right_key = first(right._keys())
-        dsk = dict(((name, i), (apply, pd.merge, [left_key, right_key],
-                                kwargs))
-                   for i, left_key in enumerate(left._keys()))
-        divisions = left.divisions
-    return DataFrame(toolz.merge(dsk, left.dask, right.dask), name,
-                     meta, divisions)
+        left_key = first(left.__dask_keys__())
+        dsk = {(name, i): (apply, pd.merge, [left_key, right_key], kwargs)
+               for i, right_key in enumerate(right.__dask_keys__())}
 
-
-def _pdconcat(dfs, axis=0, join='outer'):
-    """ Concatenate caring empty Series """
-
-    # Concat with empty Series with axis=1 will not affect to the
-    # result. Special handling is needed in each partition
-    if axis == 1:
-        # becahse dfs is a generator, once convert to list
-        dfs = list(dfs)
-
-        if join == 'outer':
-            # outer concat should keep all empty Series
-
-            # input must include one non-empty data at least
-            # because of the alignment
-            first = [df for df in dfs if len(df) > 0][0]
-
-            def _pad(base, fillby):
-                if isinstance(base, pd.Series) and len(base) == 0:
-                    # use aligned index to keep index for outer concat
-                    return pd.Series([np.nan] * len(fillby),
-                                      index=fillby.index, name=base.name)
-                else:
-                    return base
-
-            dfs = [_pad(df, first) for df in dfs]
+        if kwargs.get('right_index'):
+            divisions = right.divisions
         else:
-            # inner concat should result in empty if any input is empty
-            if any(len(df) == 0 for df in dfs):
-                dfs = [pd.DataFrame(columns=df.columns)
-                       if isinstance(df, pd.DataFrame) else
-                       pd.Series(name=df.name) for df in dfs]
+            divisions = [None for _ in right.divisions]
 
-    return pd.concat(dfs, axis=axis, join=join)
+    elif right.npartitions == 1:
+        right_key = first(right.__dask_keys__())
+        dsk = {(name, i): (apply, pd.merge, [left_key, right_key], kwargs)
+               for i, left_key in enumerate(left.__dask_keys__())}
 
+        if kwargs.get('left_index'):
+            divisions = left.divisions
+        else:
+            divisions = [None for _ in left.divisions]
 
-def concat_and_check(dfs):
-    if len(set(map(len, dfs))) != 1:
-        raise ValueError("Concatenated DataFrames of different lengths")
-    return pd.concat(dfs, axis=1)
-
-
-def concat_unindexed_dataframes(dfs):
-    name = 'concat-' + tokenize(*dfs)
-
-    dsk = {(name, i): (concat_and_check, [(df._name, i) for df in dfs])
-            for i in range(dfs[0].npartitions)}
-
-    meta = pd.concat([df._meta for df in dfs], axis=1)
-
-    return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
-                         name, meta, dfs[0].divisions)
-
-
-def concat_indexed_dataframes(dfs, axis=0, join='outer'):
-    """ Concatenate indexed dataframes together along the index """
-
-    if join not in ('inner', 'outer'):
-        raise ValueError("'join' must be 'inner' or 'outer'")
-
-    from dask.dataframe.core import _emulate
-    meta = _emulate(pd.concat, dfs, axis=axis, join=join)
-
-    dfs = _maybe_from_pandas(dfs)
-    dfs2, divisions, parts = align_partitions(*dfs)
-    empties = [df._meta for df in dfs]
-
-    parts2 = [[df if df is not None else empty
-               for df, empty in zip(part, empties)]
-              for part in parts]
-
-    name = 'concat-indexed-' + tokenize(join, *dfs)
-    dsk = dict(((name, i), (_pdconcat, part, axis, join))
-                for i, part in enumerate(parts2))
-
-    return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs2]),
-                         name, meta, divisions)
+    return new_dd_object(toolz.merge(dsk, left.dask, right.dask), name,
+                         meta, divisions)
 
 
 @wraps(pd.merge)
@@ -494,35 +410,72 @@ def merge(left, right, how='inner', on=None, left_on=None, right_on=None,
 # Concat
 ###############################################################
 
-def _concat_dfs(dfs, name, join='outer'):
-    """ Internal function to concat dask dict and DataFrame.columns """
-    dsk = dict()
+def concat_and_check(dfs):
+    if len(set(map(len, dfs))) != 1:
+        raise ValueError("Concatenated DataFrames of different lengths")
+    return pd.concat(dfs, axis=1)
+
+
+def concat_unindexed_dataframes(dfs):
+    name = 'concat-' + tokenize(*dfs)
+
+    dsk = {(name, i): (concat_and_check, [(df._name, i) for df in dfs])
+           for i in range(dfs[0].npartitions)}
+
+    meta = pd.concat([df._meta for df in dfs], axis=1)
+
+    return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
+                         name, meta, dfs[0].divisions)
+
+
+def concat_indexed_dataframes(dfs, axis=0, join='outer'):
+    """ Concatenate indexed dataframes together along the index """
+    meta = methods.concat([df._meta for df in dfs], axis=axis, join=join)
+    empties = [strip_unknown_categories(df._meta) for df in dfs]
+
+    dfs2, divisions, parts = align_partitions(*dfs)
+
+    name = 'concat-indexed-' + tokenize(join, *dfs)
+
+    parts2 = [[df if df is not None else empty
+               for df, empty in zip(part, empties)]
+              for part in parts]
+
+    dsk = dict(((name, i), (methods.concat, part, axis, join))
+               for i, part in enumerate(parts2))
+    for df in dfs2:
+        dsk.update(df.dask)
+
+    return new_dd_object(dsk, name, meta, divisions)
+
+
+def stack_partitions(dfs, divisions, join='outer'):
+    """Concatenate partitions on axis=0 by doing a simple stack"""
+    meta = methods.concat([df._meta for df in dfs], join=join)
+    empty = strip_unknown_categories(meta)
+
+    name = 'concat-{0}'.format(tokenize(*dfs))
+    dsk = {}
     i = 0
-
-    empties = [df._meta for df in dfs]
-    meta = pd.concat(empties, axis=0, join=join)
-
-    if isinstance(meta, pd.Series):
-        # in this case, input must be all Series. No need to care DataFrame.
-        columns = pd.Index([])
-    else:
-        columns = meta.columns
-        if len(columns) == 0:
-            raise ValueError('Failed to concat, no columns remain')
-
     for df in dfs:
-        if isinstance(df, DataFrame):
-            # filter DataFrame columns
-            if not columns.equals(df.columns):
-                df = df[[c for c in columns if c in df.columns]]
-        # Series must remain if output columns exist
+        dsk.update(df.dask)
+        # An error will be raised if the schemas or categories don't match. In
+        # this case we need to pass along the meta object to transform each
+        # partition, so they're all equivalent.
+        try:
+            df._meta == meta
+            match = True
+        except (ValueError, TypeError):
+            match = False
 
-        dsk = toolz.merge(dsk, df.dask)
-        for key in df._keys():
-            dsk[(name, i)] = key
+        for key in df.__dask_keys__():
+            if match:
+                dsk[(name, i)] = key
+            else:
+                dsk[(name, i)] = (methods.concat, [empty, key], 0, join)
             i += 1
 
-    return dsk, meta
+    return new_dd_object(dsk, name, meta, divisions)
 
 
 def concat(dfs, axis=0, join='outer', interleave_partitions=False):
@@ -593,102 +546,52 @@ def concat(dfs, axis=0, join='outer', interleave_partitions=False):
     dd.DataFrame<concat-..., divisions=(None, None, None, None)>
     """
     if not isinstance(dfs, list):
-        dfs = [dfs]
+        raise TypeError("dfs must be a list of DataFrames/Series objects")
     if len(dfs) == 0:
-        raise ValueError('Input must be a list longer than 0')
+        raise ValueError('No objects to concatenate')
     if len(dfs) == 1:
-        return dfs[0]
+        if axis == 1 and isinstance(dfs[0], Series):
+            return dfs[0].to_frame()
+        else:
+            return dfs[0]
 
     if join not in ('inner', 'outer'):
         raise ValueError("'join' must be 'inner' or 'outer'")
 
     axis = DataFrame._validate_axis(axis)
     dasks = [df for df in dfs if isinstance(df, _Frame)]
+    dfs = _maybe_from_pandas(dfs)
 
-    if all(df.known_divisions for df in dasks):
-        dfs = _maybe_from_pandas(dfs)
-        if axis == 1:
+    if axis == 1:
+        if all(df.known_divisions for df in dasks):
             return concat_indexed_dataframes(dfs, axis=axis, join=join)
+        elif (len(dasks) == len(dfs) and
+              all(not df.known_divisions for df in dfs) and
+              len({df.npartitions for df in dasks}) == 1):
+            warn("Concatenating dataframes with unknown divisions.\n"
+                 "We're assuming that the indexes of each dataframes are \n"
+                 "aligned. This assumption is not generally safe.")
+            return concat_unindexed_dataframes(dfs)
         else:
-            # must be converted here to check whether divisions can be
-            # concatenated
-            dfs = _maybe_from_pandas(dfs)
+            raise ValueError('Unable to concatenate DataFrame with unknown '
+                             'division specifying axis=1')
+    else:
+        if all(df.known_divisions for df in dasks):
             # each DataFrame's division must be greater than previous one
             if all(dfs[i].divisions[-1] < dfs[i + 1].divisions[0]
                    for i in range(len(dfs) - 1)):
-                name = 'concat-{0}'.format(tokenize(*dfs))
-                dsk, meta = _concat_dfs(dfs, name, join=join)
-
                 divisions = []
                 for df in dfs[:-1]:
                     # remove last to concatenate with next
                     divisions += df.divisions[:-1]
                 divisions += dfs[-1].divisions
-                return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
-                                     name, meta, divisions)
+                return stack_partitions(dfs, divisions, join=join)
+            elif interleave_partitions:
+                return concat_indexed_dataframes(dfs, join=join)
             else:
-                if interleave_partitions:
-                    return concat_indexed_dataframes(dfs, join=join)
-
-                raise ValueError('All inputs have known divisions which cannot '
-                                 'be concatenated in order. Specify '
+                raise ValueError('All inputs have known divisions which '
+                                 'cannot be concatenated in order. Specify '
                                  'interleave_partitions=True to ignore order')
-    elif (axis == 1 and
-          len(dasks) == len(dfs) and
-          all(not df.known_divisions for df in dfs) and
-          len({df.npartitions for df in dasks}) == 1):
-        warn("Concatenating dataframes with unknown divisions.\n"
-             "We're assuming that the indexes of each dataframes are aligned\n."
-             "This assumption is not generally safe.")
-        return concat_unindexed_dataframes(dfs)
-
-    else:
-        if axis == 1:
-            raise ValueError('Unable to concatenate DataFrame with unknown '
-                             'division specifying axis=1')
         else:
-            # concat will not regard Series as row
-            dfs = _maybe_from_pandas(dfs)
-            name = 'concat-{0}'.format(tokenize(*dfs))
-            dsk, meta = _concat_dfs(dfs, name, join=join)
-
             divisions = [None] * (sum([df.npartitions for df in dfs]) + 1)
-            return new_dd_object(toolz.merge(dsk, *[df.dask for df in dfs]),
-                                 name, meta, divisions)
-
-
-###############################################################
-# Append
-###############################################################
-
-
-def _append(df, other, divisions):
-    """ Internal function to append 2 dd.DataFrame/Series instances """
-    # ToDo: might be possible to merge the logic to concat,
-    token = tokenize(df, other)
-    name = '{0}-append--{1}'.format(df._token_prefix, token)
-    dsk = {}
-
-    npart = df.npartitions
-    for i in range(npart):
-        dsk[(name, i)] = (df._name, i)
-    for j in range(other.npartitions):
-        dsk[(name, npart + j)] = (other._name, j)
-    dsk = toolz.merge(dsk, df.dask, other.dask)
-    meta = df._meta.append(other._meta)
-    return new_dd_object(dsk, name, meta, divisions)
-
-
-###############################################################
-# Melt
-###############################################################
-
-def melt(frame, id_vars=None, value_vars=None, var_name=None,
-         value_name='value', col_level=None):
-
-    from dask.dataframe.core import no_default
-
-    return frame.map_partitions(pd.melt, meta=no_default, id_vars=id_vars,
-                                value_vars=value_vars,
-                                var_name=var_name, value_name=value_name,
-                                col_level=col_level, token='melt')
+            return stack_partitions(dfs, divisions, join=join)
