@@ -76,9 +76,10 @@ import pandas as pd
 
 from toolz import merge, merge_sorted, take
 
-from ..utils import different_seeds
+from ..utils import random_state_data
 from ..base import tokenize
-from .core import Index, Series
+from .core import Series
+from .utils import is_categorical_dtype
 from dask.compatibility import zip
 
 
@@ -186,15 +187,15 @@ def tree_groups(N, num_groups):
     group_size = N // num_groups
     dx = num_groups
     dy = N - group_size * num_groups
-    D = 2*dy - dx
+    D = 2 * dy - dx
     rv = []
     for _ in range(num_groups):
         if D < 0:
             rv.append(group_size)
         else:
             rv.append(group_size + 1)
-            D -= 2*dx
-        D += 2*dy
+            D -= 2 * dx
+        D += 2 * dy
     return rv
 
 
@@ -227,7 +228,7 @@ def create_merge_tree(func, keys, token):
         groups = tree_groups(prev_width, width)
         keys = [(token, level, i) for i in range(width)]
         rv.update((key, (func, list(take(num, prev_keys))))
-                   for num, key in zip(groups, keys))
+                  for num, key in zip(groups, keys))
         prev_width = width
         prev_keys = iter(keys)
         level += 1
@@ -306,10 +307,18 @@ def process_val_weights(vals_and_weights, npartitions, dtype_info):
     aren't enough unique values in the column.  Increasing ``upsample``
     keyword argument in ``df.set_index`` may help.
     """
+    dtype, info = dtype_info
+
+    if not vals_and_weights:
+        try:
+            return np.array(None, dtype=dtype)
+        except Exception:
+            # dtype does not support None value so allow it to change
+            return np.array(None, dtype=np.float_)
+
     vals, weights = vals_and_weights
     vals = np.array(vals)
     weights = np.array(weights)
-    dtype, info = dtype_info
 
     # We want to create exactly `npartition` number of groups of `vals` that
     # are approximately the same weight and non-empty if possible.  We use a
@@ -363,7 +372,7 @@ def process_val_weights(vals_and_weights, npartitions, dtype_info):
         rv = np.concatenate([trimmed, jumbo_vals])
         rv.sort()
 
-    if str(dtype) == 'category':
+    if is_categorical_dtype(dtype):
         rv = pd.Categorical.from_codes(rv, info[0], info[1])
     elif 'datetime64' in str(dtype):
         rv = pd.DatetimeIndex(rv, dtype=dtype)
@@ -372,7 +381,7 @@ def process_val_weights(vals_and_weights, npartitions, dtype_info):
     return rv
 
 
-def percentiles_summary(df, num_old, num_new, upsample=1.0, random_state=None):
+def percentiles_summary(df, num_old, num_new, upsample, state):
     """Summarize data using percentiles and derived weights.
 
     These summaries can be merged, compressed, and converted back into
@@ -394,10 +403,11 @@ def percentiles_summary(df, num_old, num_new, upsample=1.0, random_state=None):
     length = len(df)
     if length == 0:
         return ()
+    random_state = np.random.RandomState(state)
     qs = sample_percentiles(num_old, num_new, length, upsample, random_state)
     data = df.values
     interpolation = 'linear'
-    if str(data.dtype) == 'category':
+    if is_categorical_dtype(data):
         data = data.codes
         interpolation = 'nearest'
     vals = _percentile(data, qs, interpolation=interpolation)
@@ -409,7 +419,7 @@ def percentiles_summary(df, num_old, num_new, upsample=1.0, random_state=None):
 
 def dtype_info(df):
     info = None
-    if str(df.dtype) == 'category':
+    if is_categorical_dtype(df):
         data = df.values
         info = (data.categories, data.ordered)
     return df.dtype, info
@@ -419,31 +429,25 @@ def partition_quantiles(df, npartitions, upsample=1.0, random_state=None):
     """ Approximate quantiles of Series used for repartitioning
     """
     assert isinstance(df, Series)
-
-    qs = np.linspace(0, 1, npartitions + 1)
     # currently, only Series has quantile method
     # Index.quantile(list-like) must be pd.Series, not pd.Index
-    df_name = df.name
-    merge_type = lambda v: pd.Series(v, index=qs, name=df_name)
-    return_type = df._constructor
-    if issubclass(return_type, Index):
-        return_type = Series
+    return_type = Series
 
-    new_divisions = [0.0, 1.0]
+    qs = np.linspace(0, 1, npartitions + 1)
     token = tokenize(df, qs, upsample)
     if random_state is None:
         random_state = hash(token) % np.iinfo(np.int32).max
-    seeds = different_seeds(df.npartitions, random_state)
+    state_data = random_state_data(df.npartitions, random_state)
 
-    df_keys = df._keys()
+    df_keys = df.__dask_keys__()
 
     name0 = 're-quantiles-0-' + token
     dtype_dsk = {(name0, 0): (dtype_info, df_keys[0])}
 
     name1 = 're-quantiles-1-' + token
-    val_dsk = dict(((name1, i), (percentiles_summary, key, df.npartitions,
-                                 npartitions, upsample, seeds[i]))
-                   for i, key in enumerate(df_keys))
+    val_dsk = {(name1, i): (percentiles_summary, key, df.npartitions,
+                            npartitions, upsample, state)
+               for i, (state, key) in enumerate(zip(state_data, df_keys))}
 
     name2 = 're-quantiles-2-' + token
     merge_dsk = create_merge_tree(merge_and_compress_summaries, sorted(val_dsk), name2)
@@ -454,8 +458,9 @@ def partition_quantiles(df, npartitions, upsample=1.0, random_state=None):
     merged_key = max(merge_dsk)
 
     name3 = 're-quantiles-3-' + token
-    last_dsk = {(name3, 0): (merge_type, (process_val_weights, merged_key,
-                                          npartitions, (name0, 0)))}
+    last_dsk = {(name3, 0): (pd.Series, (process_val_weights, merged_key,
+                                         npartitions, (name0, 0)), qs, None, df.name)}
 
     dsk = merge(df.dask, dtype_dsk, val_dsk, merge_dsk, last_dsk)
+    new_divisions = [0.0, 1.0]
     return return_type(dsk, name3, df._meta, new_divisions)
