@@ -49,6 +49,8 @@ for df in frames:
                              y=df.y.cat.set_categories(list('abc'))))
 frames3 = [i.set_index(i.y) for i in frames]
 frames4 = [i.set_index(i.y) for i in frames2]
+frames5 = [i.set_index([i.y, i.x]) for i in frames]
+frames6 = [i.set_index([i.y, i.x]) for i in frames2]
 
 
 def test_concat_unions_categoricals():
@@ -78,6 +80,13 @@ def test_concat_unions_categoricals():
     tm.assert_series_equal(_concat([i.x for i in frames3]),
                            pd.concat([i.x for i in frames4]))
 
+    # MultiIndex with Categorical Index
+    tm.assert_index_equal(_concat([i.index for i in frames5]),
+                          pd.concat([i for i in frames6]).index)
+
+    # DataFrame, MultiIndex with CategoricalIndex
+    tm.assert_frame_equal(_concat(frames5), pd.concat(frames6))
+
 
 def test_unknown_categoricals():
     ddf = dd.DataFrame({('unknown', i): df for (i, df) in enumerate(frames)},
@@ -193,7 +202,7 @@ def test_categorize_index():
 @pytest.mark.parametrize('shuffle', ['disk', 'tasks'])
 def test_categorical_set_index(shuffle):
     df = pd.DataFrame({'x': [1, 2, 3, 4], 'y': ['a', 'b', 'b', 'c']})
-    df['y'] = df.y.astype('category', ordered=True)
+    df['y'] = pd.Categorical(df['y'], categories=['a', 'b', 'c'], ordered=True)
     a = dd.from_pandas(df, npartitions=2)
 
     with dask.set_options(get=dask.get, shuffle=shuffle):
@@ -214,6 +223,19 @@ def test_categorical_set_index(shuffle):
         assert list(sorted(d2.index.compute())) == ['b', 'b', 'c']
 
 
+@pytest.mark.parametrize('npartitions', [1, 4])
+def test_repartition_on_categoricals(npartitions):
+    df = pd.DataFrame({'x': range(10), 'y': list('abababcbcb')})
+    ddf = dd.from_pandas(df, npartitions=2)
+    ddf['y'] = ddf['y'].astype('category')
+    ddf2 = ddf.repartition(npartitions=npartitions)
+
+    df = df.copy()
+    df['y'] = df['y'].astype('category')
+    assert_eq(df, ddf)
+    assert_eq(df, ddf2)
+
+
 def test_categorical_accessor_presence():
     df = pd.DataFrame({'x': list('a' * 5 + 'b' * 5 + 'c' * 5), 'y': range(15)})
     df.x = df.x.astype('category')
@@ -320,3 +342,16 @@ class TestCategoricalAccessor:
         res = db.compute()
         tm.assert_index_equal(db.cat.categories, get_cat(res).categories)
         assert_array_index_eq(db.cat.codes, get_cat(res).codes)
+
+    def test_categorical_string_ops(self):
+        a = pd.Series(['a', 'a', 'b'], dtype='category')
+        da = dd.from_pandas(a, 2)
+        result = da.str.upper()
+        expected = a.str.upper()
+        assert_eq(result, expected)
+
+    def test_categorical_non_string_raises(self):
+        a = pd.Series([1, 2, 3], dtype='category')
+        da = dd.from_pandas(a, 2)
+        with pytest.raises(AttributeError):
+            da.str.upper()
