@@ -8,11 +8,12 @@ import numpy as np
 import pytest
 
 import dask
-from dask.utils import ignoring, put_lines
 import dask.dataframe as dd
+from dask.base import compute_as_if_collection
+from dask.utils import ignoring, put_lines
 
 from dask.dataframe.core import repartition_divisions, aca, _concat, Scalar
-from dask.dataframe.methods import boundary_slice
+from dask.dataframe import methods
 from dask.dataframe.utils import (assert_eq, make_meta, assert_max_deps,
                                   PANDAS_VERSION)
 
@@ -626,10 +627,14 @@ def test_unique():
 
 
 def test_isin():
+    # Series test
     assert_eq(d.a.isin([0, 1, 2]), full.a.isin([0, 1, 2]))
     assert_eq(d.a.isin(pd.Series([0, 1, 2])),
               full.a.isin(pd.Series([0, 1, 2])))
 
+    # DataFrame test
+    assert_eq(d.isin([0, 1, 2]), full.isin([0, 1, 2]))
+
 
 def test_len():
     assert len(d) == len(full)
@@ -685,16 +690,26 @@ def test_quantile():
     assert 4 < result < 6
 
 
+def test_quantile_missing():
+    df = pd.DataFrame({"A": [0, np.nan, 2]})
+    ddf = dd.from_pandas(df, 2)
+    expected = df.quantile()
+    result = ddf.quantile()
+    assert_eq(result, expected)
+
+    expected = df.A.quantile()
+    result = ddf.A.quantile()
+    assert_eq(result, expected)
+
+
 def test_empty_quantile():
     result = d.b.quantile([])
     exp = full.b.quantile([])
     assert result.divisions == (None, None)
 
-    # because of a pandas bug, name is not preserved
-    # https://github.com/pydata/pandas/pull/10881
     assert result.name == 'b'
     assert result.compute().name == 'b'
-    assert_eq(result, exp, check_names=False)
+    assert_eq(result, exp)
 
 
 def test_dataframe_quantile():
@@ -712,6 +727,7 @@ def test_dataframe_quantile():
 
     result = result.compute()
     assert isinstance(result, pd.Series)
+    assert result.name == 0.5
     tm.assert_index_equal(result.index, pd.Index(['A', 'X', 'B']))
     assert (result > pd.Series([16, 36, 26], index=['A', 'X', 'B'])).all()
     assert (result < pd.Series([17, 37, 27], index=['A', 'X', 'B'])).all()
@@ -745,15 +761,18 @@ def test_assign():
     res = d.assign(c=1,
                    d='string',
                    e=d.a.sum(),
-                   f=d.a + d.b)
+                   f=d.a + d.b,
+                   g=lambda x: x.a + x.b)
     res_unknown = d_unknown.assign(c=1,
                                    d='string',
                                    e=d_unknown.a.sum(),
-                                   f=d_unknown.a + d_unknown.b)
+                                   f=d_unknown.a + d_unknown.b,
+                                   g=lambda x: x.a + x.b)
     sol = full.assign(c=1,
                       d='string',
                       e=full.a.sum(),
-                      f=full.a + full.b)
+                      f=full.a + full.b,
+                      g=lambda x: x.a + x.b)
     assert_eq(res, sol)
     assert_eq(res_unknown, sol)
 
@@ -1013,7 +1032,8 @@ def test_repartition():
         """Check data is split properly"""
         keys = [k for k in d.dask if k[0].startswith('repartition-split')]
         keys = sorted(keys)
-        sp = pd.concat([d._get(d.dask, k) for k in keys])
+        sp = pd.concat([compute_as_if_collection(dd.DataFrame, d.dask, k)
+                        for k in keys])
         assert_eq(orig, sp)
         assert_eq(orig, d)
 
@@ -1024,7 +1044,8 @@ def test_repartition():
     b = a.repartition(divisions=[10, 20, 50, 60])
     assert b.divisions == (10, 20, 50, 60)
     assert_eq(a, b)
-    assert_eq(a._get(b.dask, (b._name, 0)), df.iloc[:1])
+    assert_eq(compute_as_if_collection(dd.DataFrame, b.dask, (b._name, 0)),
+              df.iloc[:1])
 
     for div in [[20, 60], [10, 50], [1],   # first / last element mismatch
                 [0, 60], [10, 70],   # do not allow to expand divisions by default
@@ -1096,17 +1117,17 @@ def test_repartition():
 
 def test_repartition_divisions():
     result = repartition_divisions([0, 6], [0, 6, 6], 'a', 'b', 'c')
-    assert result == {('b', 0): (boundary_slice, ('a', 0), 0, 6, False),
-                      ('b', 1): (boundary_slice, ('a', 0), 6, 6, True),
+    assert result == {('b', 0): (methods.boundary_slice, ('a', 0), 0, 6, False),
+                      ('b', 1): (methods.boundary_slice, ('a', 0), 6, 6, True),
                       ('c', 0): ('b', 0),
                       ('c', 1): ('b', 1)}
 
     result = repartition_divisions([1, 3, 7], [1, 4, 6, 7], 'a', 'b', 'c')
-    assert result == {('b', 0): (boundary_slice, ('a', 0), 1, 3, False),
-                      ('b', 1): (boundary_slice, ('a', 1), 3, 4, False),
-                      ('b', 2): (boundary_slice, ('a', 1), 4, 6, False),
-                      ('b', 3): (boundary_slice, ('a', 1), 6, 7, True),
-                      ('c', 0): (pd.concat, [('b', 0), ('b', 1)]),
+    assert result == {('b', 0): (methods.boundary_slice, ('a', 0), 1, 3, False),
+                      ('b', 1): (methods.boundary_slice, ('a', 1), 3, 4, False),
+                      ('b', 2): (methods.boundary_slice, ('a', 1), 4, 6, False),
+                      ('b', 3): (methods.boundary_slice, ('a', 1), 6, 7, True),
+                      ('c', 0): (methods.concat, [('b', 0), ('b', 1)]),
                       ('c', 1): ('b', 2),
                       ('c', 2): ('b', 3)}
 
@@ -1139,7 +1160,7 @@ def test_repartition_npartitions(use_index, n, k, dtype, transform):
     b = a.repartition(npartitions=k)
     assert_eq(a, b)
     assert b.npartitions == k
-    parts = dask.get(b.dask, b._keys())
+    parts = dask.get(b.dask, b.__dask_keys__())
     assert all(map(len, parts))
 
 
@@ -1170,9 +1191,9 @@ def test_repartition_object_index():
 
 @pytest.mark.slow
 @pytest.mark.parametrize('npartitions', [1, 20, 243])
-@pytest.mark.parametrize('freq', ['1D', '7D', '28h'])
-@pytest.mark.parametrize('end', ['2000-04-15', '2000-04-15 12:37:01'])
-@pytest.mark.parametrize('start', ['2000-01-01', '2000-01-01 12:30:00'])
+@pytest.mark.parametrize('freq', ['1D', '7D', '28h', '1h'])
+@pytest.mark.parametrize('end', ['2000-04-15', '2000-04-15 12:37:01', '2000-01-01 12:37:00'])
+@pytest.mark.parametrize('start', ['2000-01-01', '2000-01-01 12:30:00', '2000-01-01 12:30:00'])
 def test_repartition_freq(npartitions, freq, start, end):
     start = pd.Timestamp(start)
     end = pd.Timestamp(end)
@@ -1286,6 +1307,14 @@ def test_ffill_bfill():
     assert_eq(ddf.bfill(axis=1), df.bfill(axis=1))
 
 
+def test_fillna_series_types():
+    # https://github.com/dask/dask/issues/2809
+    df = pd.DataFrame({"A": [1, np.nan, 3], "B": [1, np.nan, 3]})
+    ddf = dd.from_pandas(df, npartitions=2)
+    fill_value = pd.Series([1, 10], index=['A', 'C'])
+    assert_eq(ddf.fillna(fill_value), df.fillna(fill_value))
+
+
 def test_sample():
     df = pd.DataFrame({'x': [1, 2, 3, 4, None, 6], 'y': list('abdabd')},
                       index=[10, 20, 30, 40, 50, 60])
@@ -1332,7 +1361,7 @@ def test_datetime_accessor():
 
 
 def test_str_accessor():
-    df = pd.DataFrame({'x': ['a', 'b', 'c', 'D'], 'y': [1, 2, 3, 4]},
+    df = pd.DataFrame({'x': ['abc', 'bcd', 'cdef', 'DEFG'], 'y': [1, 2, 3, 4]},
                       index=['e', 'f', 'g', 'H'])
 
     a = dd.from_pandas(df, 2, sort=False)
@@ -1341,6 +1370,10 @@ def test_str_accessor():
     assert 'str' not in dir(a.y)
     assert not hasattr(a.y, 'str')
 
+    # not implemented methods don't show up
+    assert 'get_dummies' not in dir(a.x.str)
+    assert not hasattr(a.x.str, 'get_dummies')
+
     assert 'upper' in dir(a.x.str)
     assert_eq(a.x.str.upper(), df.x.str.upper())
     assert set(a.x.str.upper().dask) == set(a.x.str.upper().dask)
@@ -1365,6 +1398,9 @@ def test_str_accessor():
         assert_eq(a.x.str.contains('a', regex=regex), df.x.str.contains('a', regex=regex))
         assert set(a.x.str.contains('a', regex=regex).dask) == set(a.x.str.contains('a', regex=regex).dask)
 
+    assert_eq(df.x.str[:2], df.x.str[:2])
+    assert_eq(a.x.str[1], a.x.str[1])
+
 
 def test_empty_max():
     meta = make_meta({'x': 'i8'})
@@ -1394,9 +1430,10 @@ def test_eval():
 
         # catch FutureWarning from pandas about assignment in eval
         with pytest.warns(None):
-            if p.eval('z = x + y', inplace=None) is None:
-                with pytest.raises(NotImplementedError):
-                    d.eval('z = x + y', inplace=None)
+            if PANDAS_VERSION < '0.21.0':
+                if p.eval('z = x + y', inplace=None) is None:
+                    with pytest.raises(NotImplementedError):
+                        d.eval('z = x + y', inplace=None)
 
 
 @pytest.mark.parametrize('include, exclude', [
@@ -1895,7 +1932,7 @@ def test_cov_corr_meta():
 
 @pytest.mark.slow
 def test_cov_corr_stable():
-    df = pd.DataFrame(np.random.random((20000000, 2)) * 2 - 1, columns=['a', 'b'])
+    df = pd.DataFrame(np.random.uniform(-1, 1, (20000000, 2)), columns=['a', 'b'])
     ddf = dd.from_pandas(df, npartitions=50)
     assert_eq(ddf.cov(split_every=8), df.cov())
     assert_eq(ddf.corr(split_every=8), df.corr())
@@ -2057,11 +2094,15 @@ def test_astype():
 
 def test_astype_categoricals():
     df = pd.DataFrame({'x': ['a', 'b', 'c', 'b', 'c'],
-                       'y': [1, 2, 3, 4, 5]})
+                       'y': ['x', 'y', 'z', 'x', 'y'],
+                       'z': [1, 2, 3, 4, 5]})
+    df = df.astype({'y': 'category'})
     ddf = dd.from_pandas(df, 2)
+    assert ddf.y.cat.known
 
     ddf2 = ddf.astype({'x': 'category'})
     assert not ddf2.x.cat.known
+    assert ddf2.y.cat.known
     assert ddf2.x.dtype == 'category'
     assert ddf2.compute().x.dtype == 'category'
 
@@ -2071,6 +2112,36 @@ def test_astype_categoricals():
     assert dx.compute().dtype == 'category'
 
 
+@pytest.mark.skipif(PANDAS_VERSION < '0.21.0',
+                    reason="No CategoricalDtype with categories")
+def test_astype_categoricals_known():
+    df = pd.DataFrame({'x': ['a', 'b', 'c', 'b', 'c'],
+                       'y': ['x', 'y', 'z', 'y', 'z'],
+                       'z': ['b', 'b', 'b', 'c', 'b'],
+                       'other': [1, 2, 3, 4, 5]})
+    ddf = dd.from_pandas(df, 2)
+
+    abc = pd.api.types.CategoricalDtype(['a', 'b', 'c'])
+    category = pd.api.types.CategoricalDtype()
+
+    # DataFrame
+    ddf2 = ddf.astype({'x': abc,
+                       'y': category,
+                       'z': 'category',
+                       'other': 'f8'})
+
+    for col, known in [('x', True), ('y', False), ('z', False)]:
+        x = getattr(ddf2, col)
+        assert pd.api.types.is_categorical_dtype(x.dtype)
+        assert x.cat.known == known
+
+    # Series
+    for dtype, known in [('category', False), (category, False), (abc, True)]:
+        dx2 = ddf.x.astype(dtype)
+        assert pd.api.types.is_categorical_dtype(dx2.dtype)
+        assert dx2.cat.known == known
+
+
 def test_groupby_callable():
     a = pd.DataFrame({'x': [1, 2, 3, None], 'y': [10, 20, 30, 40]},
                      index=[1, 2, 3, 4])
@@ -2184,13 +2255,14 @@ def test_categorize_info():
     # Verbose=False
     buf = StringIO()
     ddf.info(buf=buf, verbose=True)
-    assert buf.getvalue() == unicode("<class 'dask.dataframe.core.DataFrame'>\n"
-                                     "Int64Index: 4 entries, 0 to 3\n"
-                                     "Data columns (total 3 columns):\n"
-                                     "x    4 non-null int64\n"
-                                     "y    4 non-null category\n"
-                                     "z    4 non-null object\n"
-                                     "dtypes: category(1), object(1), int64(1)")
+    expected = unicode("<class 'dask.dataframe.core.DataFrame'>\n"
+                       "Int64Index: 4 entries, 0 to 3\n"
+                       "Data columns (total 3 columns):\n"
+                       "x    4 non-null int64\n"
+                       "y    4 non-null category\n"
+                       "z    4 non-null object\n"
+                       "dtypes: category(1), object(1), int64(1)")
+    assert buf.getvalue() == expected
 
 
 def test_gh_1301():
@@ -2242,6 +2314,13 @@ def test_attribute_assignment():
     assert_eq(ddf, df.assign(y=df.x + df.y))
 
 
+def test_setitem_triggering_realign():
+    a = dd.from_pandas(pd.DataFrame({"A": range(12)}), npartitions=3)
+    b = dd.from_pandas(pd.Series(range(12), name='B'), npartitions=4)
+    a['C'] = b
+    assert len(a) == 12
+
+
 def test_inplace_operators():
     df = pd.DataFrame({'x': [1, 2, 3, 4, 5],
                        'y': [1., 2., 3., 4., 5.]})
@@ -2295,6 +2374,33 @@ def test_idxmaxmin(idx, skipna):
             ddf.a.idxmin(skipna=skipna, split_every=2)._name)
 
 
+def test_idxmaxmin_empty_partitions():
+    df = pd.DataFrame({'a': [1, 2, 3],
+                       'b': [1.5, 2, 3],
+                       'c': [np.NaN] * 3,
+                       'd': [1, 2, np.NaN]})
+    empty = df.iloc[:0]
+
+    ddf = dd.concat([dd.from_pandas(df, npartitions=1)] +
+                    [dd.from_pandas(empty, npartitions=1)] * 10)
+
+    for skipna in [True, False]:
+        assert_eq(ddf.idxmin(skipna=skipna, split_every=3),
+                  df.idxmin(skipna=skipna))
+
+    assert_eq(ddf[['a', 'b', 'd']].idxmin(skipna=skipna, split_every=3),
+              df[['a', 'b', 'd']].idxmin(skipna=skipna))
+
+    assert_eq(ddf.b.idxmax(split_every=3), df.b.idxmax())
+
+    # Completely empty raises
+    ddf = dd.concat([dd.from_pandas(empty, npartitions=1)] * 10)
+    with pytest.raises(ValueError):
+        ddf.idxmax().compute()
+    with pytest.raises(ValueError):
+        ddf.b.idxmax().compute()
+
+
 def test_getitem_meta():
     data = {'col1': ['a', 'a', 'b'],
             'col2': [0, 1, 0]}
@@ -2437,7 +2543,7 @@ def test_hash_split_unique(npartitions, split_every, split_out):
 
     dropped = ds.unique(split_every=split_every, split_out=split_out)
 
-    dsk = dropped._optimize(dropped.dask, dropped._keys())
+    dsk = dropped.__dask_optimize__(dropped.dask, dropped.__dask_keys__())
     from dask.core import get_deps
     dependencies, dependents = get_deps(dsk)
 
@@ -2566,6 +2672,19 @@ def test_to_datetime():
               dd.to_datetime(ds, infer_datetime_format=True))
 
 
+def test_to_timedelta():
+    s = pd.Series(range(10))
+    ds = dd.from_pandas(s, npartitions=2)
+
+    assert_eq(pd.to_timedelta(s), dd.to_timedelta(ds))
+    assert_eq(pd.to_timedelta(s, unit='h'), dd.to_timedelta(ds, unit='h'))
+
+    s = pd.Series([1, 2, 'this will error'])
+    ds = dd.from_pandas(s, npartitions=2)
+    assert_eq(pd.to_timedelta(s, errors='coerce'),
+              dd.to_timedelta(ds, errors='coerce'))
+
+
 @pytest.mark.parametrize('drop', [0, 9])
 def test_slice_on_filtered_boundary(drop):
     # https://github.com/dask/dask/issues/2211
@@ -2583,23 +2702,23 @@ def test_slice_on_filtered_boundary(drop):
 def test_boundary_slice_nonmonotonic():
     x = np.array([-1, -2, 2, 4, 3])
     df = pd.DataFrame({"B": range(len(x))}, index=x)
-    result = boundary_slice(df, 0, 4)
+    result = methods.boundary_slice(df, 0, 4)
     expected = df.iloc[2:]
     tm.assert_frame_equal(result, expected)
 
-    result = boundary_slice(df, -1, 4)
+    result = methods.boundary_slice(df, -1, 4)
     expected = df.drop(-2)
     tm.assert_frame_equal(result, expected)
 
-    result = boundary_slice(df, -2, 3)
+    result = methods.boundary_slice(df, -2, 3)
     expected = df.drop(4)
     tm.assert_frame_equal(result, expected)
 
-    result = boundary_slice(df, -2, 3.5)
+    result = methods.boundary_slice(df, -2, 3.5)
     expected = df.drop(4)
     tm.assert_frame_equal(result, expected)
 
-    result = boundary_slice(df, -2, 4)
+    result = methods.boundary_slice(df, -2, 4)
     expected = df
     tm.assert_frame_equal(result, expected)
 
@@ -2620,7 +2739,7 @@ def test_boundary_slice_nonmonotonic():
 def test_with_boundary(start, stop, right_boundary, left_boundary, drop):
     x = np.array([-1, -2, 2, 4, 3])
     df = pd.DataFrame({"B": range(len(x))}, index=x)
-    result = boundary_slice(df, start, stop, right_boundary, left_boundary)
+    result = methods.boundary_slice(df, start, stop, right_boundary, left_boundary)
     expected = df.drop(drop)
     tm.assert_frame_equal(result, expected)
 
@@ -2641,7 +2760,7 @@ def test_with_boundary(start, stop, right_boundary, left_boundary, drop):
 ])
 def test_boundary_slice_same(index, left, right):
     df = pd.DataFrame({"A": range(len(index))}, index=index)
-    result = boundary_slice(df, left, right)
+    result = methods.boundary_slice(df, left, right)
     tm.assert_frame_equal(result, df)
 
 
@@ -2652,3 +2771,14 @@ def test_better_errors_object_reductions():
     with pytest.raises(ValueError) as err:
         ds.mean()
     assert str(err.value) == "`mean` not supported with object series"
+
+
+def test_sample_empty_partitions():
+    @dask.delayed
+    def make_df(n):
+        return pd.DataFrame(np.zeros((n, 4)), columns=list('abcd'))
+    ddf = dd.from_delayed([make_df(0), make_df(100), make_df(0)])
+    ddf2 = ddf.sample(frac=0.2)
+    # smoke test sample on empty partitions
+    res = ddf2.compute()
+    assert res.dtypes.equals(ddf2.dtypes)
