@@ -1,4 +1,3 @@
-
 import numpy as np
 import pandas as pd
 import pandas.util.testing as tm
@@ -6,7 +5,7 @@ import pytest
 
 import dask.dataframe as dd
 
-from dask.dataframe.utils import assert_eq, PANDAS_ge_0190
+from dask.dataframe.utils import assert_eq, make_meta
 
 
 @pytest.mark.parametrize('data', [
@@ -28,24 +27,21 @@ def test_get_dummies_object():
     df = pd.DataFrame({'a': pd.Categorical([1, 2, 3, 4, 4, 3, 2, 1]),
                        'b': list('abcdabcd'),
                        'c': pd.Categorical(list('abcdabcd'))})
-    # exclude object columns
+    ddf = dd.from_pandas(df, 2)
+
+    # Explicitly exclude object columns
     exp = pd.get_dummies(df, columns=['a', 'c'])
-
-    ddf = dd.from_pandas(df, 2)
-    res = dd.get_dummies(ddf)
+    res = dd.get_dummies(ddf, columns=['a', 'c'])
     assert_eq(res, exp)
     tm.assert_index_equal(res.columns, exp.columns)
 
-    exp = pd.get_dummies(df, columns=['a'])
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf)
 
-    ddf = dd.from_pandas(df, 2)
-    res = dd.get_dummies(ddf, columns=['a'])
-    assert_eq(res, exp)
-    tm.assert_index_equal(res.columns, exp.columns)
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf.b)
 
-    # cannot target object columns
-    msg = 'target columns must have category dtype'
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(NotImplementedError):
         dd.get_dummies(ddf, columns=['b'])
 
 
@@ -83,18 +79,32 @@ def test_get_dummies_kwargs():
     tm.assert_index_equal(res.columns, pd.Index([1, 2, 3, 5, np.nan]))
 
     msg = 'sparse=True is not supported'
-    with tm.assertRaisesRegexp(NotImplementedError, msg):
+    with pytest.raises(NotImplementedError) as err:
         dd.get_dummies(ds, sparse=True)
+    assert msg in str(err.value)
 
 
 def test_get_dummies_errors():
-    msg = 'data must have category dtype'
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(NotImplementedError):
         # not Categorical
         s = pd.Series([1, 1, 1, 2, 2, 1, 3, 4])
         ds = dd.from_pandas(s, 2)
         dd.get_dummies(ds)
 
+    # unknown categories
+    df = pd.DataFrame({'x': list('abcbc'), 'y': list('bcbcb')})
+    ddf = dd.from_pandas(df, npartitions=2)
+    ddf._meta = make_meta({'x': 'category', 'y': 'category'})
+
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf)
+
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf, columns=['x', 'y'])
+
+    with pytest.raises(NotImplementedError):
+        dd.get_dummies(ddf.x)
+
 
 @pytest.mark.parametrize('aggfunc', ['mean', 'sum', 'count'])
 def test_pivot_table(aggfunc):
@@ -111,11 +121,7 @@ def test_pivot_table(aggfunc):
         # dask result cannot be int64 dtype depending on divisions because of NaN
         exp = exp.astype(np.float64)
 
-    if PANDAS_ge_0190:
-        assert_eq(res, exp)
-    else:
-        # because of a pandas 0.18.x bug, categorical dtype is not preserved
-        assert_eq(res, exp, check_names=False, check_column_type=False)
+    assert_eq(res, exp)
 
     # method
     res = ddf.pivot_table(index='A', columns='C', values='B',
@@ -125,11 +131,7 @@ def test_pivot_table(aggfunc):
     if aggfunc == 'count':
         # dask result cannot be int64 dtype depending on divisions because of NaN
         exp = exp.astype(np.float64)
-    if PANDAS_ge_0190:
-        assert_eq(res, exp)
-    else:
-        # because of a pandas 0.18.x bug, categorical dtype is not preserved
-        assert_eq(res, exp, check_names=False, check_column_type=False)
+    assert_eq(res, exp)
 
 
 def test_pivot_table_dtype():
@@ -148,11 +150,7 @@ def test_pivot_table_dtype():
     exp = pd.pivot_table(df, index='A', columns='B',
                          values='C', aggfunc='count').astype(np.float64)
 
-    if PANDAS_ge_0190:
-        assert_eq(res, exp)
-    else:
-        # because of a pandas 0.18.x bug, categorical dtype is not preserved
-        assert_eq(res, exp, check_names=False, check_column_type=False)
+    assert_eq(res, exp)
 
 
 def test_pivot_table_errors():
@@ -162,26 +160,39 @@ def test_pivot_table_errors():
     ddf = dd.from_pandas(df, 2)
 
     msg = "'index' must be the name of an existing column"
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError) as err:
         dd.pivot_table(ddf, index=['A'], columns='C', values='B')
+    assert msg in str(err.value)
     msg = "'columns' must be the name of an existing column"
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError) as err:
         dd.pivot_table(ddf, index='A', columns=['C'], values='B')
+    assert msg in str(err.value)
     msg = "'values' must be the name of an existing column"
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError) as err:
         dd.pivot_table(ddf, index='A', columns='C', values=['B'])
+    assert msg in str(err.value)
 
     msg = "aggfunc must be either 'mean', 'sum' or 'count'"
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError) as err:
         dd.pivot_table(ddf, index='A', columns='C', values='B', aggfunc=['sum'])
+    assert msg in str(err.value)
 
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError) as err:
         dd.pivot_table(ddf, index='A', columns='C', values='B', aggfunc='xx')
+    assert msg in str(err.value)
+
+    # unknown categories
+    ddf._meta = make_meta({'A': object, 'B': float, 'C': 'category'})
+    msg = "'columns' must have known categories"
+    with pytest.raises(ValueError) as err:
+        dd.pivot_table(ddf, index='A', columns='C', values=['B'])
+    assert msg in str(err.value)
 
     df = pd.DataFrame({'A': np.random.choice(list('abc'), size=10),
                        'B': np.random.randn(10),
                        'C': np.random.choice(list('abc'), size=10)})
     ddf = dd.from_pandas(df, 2)
     msg = "'columns' must be category dtype"
-    with tm.assertRaisesRegexp(ValueError, msg):
+    with pytest.raises(ValueError) as err:
         dd.pivot_table(ddf, index='A', columns='C', values='B')
+    assert msg in str(err.value)
