@@ -21,6 +21,7 @@ except ImportError:
 
 from ..core import get_deps
 from ..local import get_sync
+from ..utils import asciitable
 
 
 PANDAS_VERSION = LooseVersion(pd.__version__)
@@ -284,7 +285,7 @@ def make_meta(x, index=None):
         try:
             dtype = np.dtype(x)
             return _scalar_from_dtype(dtype)
-        except:
+        except Exception:
             # Continue on to next check
             pass
 
@@ -426,6 +427,71 @@ def meta_nonempty(x):
                         "got {0}".format(type(x).__name__))
 
 
+def check_meta(x, meta, funcname=None, numeric_equal=True):
+    """Check that the dask metadata matches the result.
+
+    If metadata matches, ``x`` is passed through unchanged. A nice error is
+    raised if metadata doesn't match.
+
+    Parameters
+    ----------
+    x : DataFrame, Series, or Index
+    meta : DataFrame, Series, or Index
+        The expected metadata that ``x`` should match
+    funcname : str, optional
+        The name of the function in which the metadata was specified. If
+        provided, the function name will be included in the error message to be
+        more helpful to users.
+    numeric_equal : bool, optionl
+        If True, integer and floating dtypes compare equal. This is useful due
+        to panda's implicit conversion of integer to floating upon encountering
+        missingness, which is hard to infer statically.
+    """
+    eq_types = {'i', 'f'} if numeric_equal else {}
+
+    def equal_dtypes(a, b):
+        if is_categorical_dtype(a) != is_categorical_dtype(b):
+            return False
+        if (a is '-' or b is '-'):
+            return False
+        if is_categorical_dtype(a) and is_categorical_dtype(b):
+            # Pandas 0.21 CategoricalDtype compat
+            if (PANDAS_VERSION >= '0.21.0' and
+                    (UNKNOWN_CATEGORIES in a.categories or
+                     UNKNOWN_CATEGORIES in b.categories)):
+                return True
+            return a == b
+        return (a.kind in eq_types and b.kind in eq_types) or (a == b)
+
+    if not isinstance(meta, (pd.Series, pd.Index, pd.DataFrame)):
+        raise TypeError("Expected partition to be DataFrame, Series, or "
+                        "Index, got `%s`" % type(meta).__name__)
+
+    if type(x) != type(meta):
+        errmsg = ("Expected partition of type `%s` but got "
+                  "`%s`" % (type(meta).__name__, type(x).__name__))
+    elif isinstance(meta, pd.DataFrame):
+        dtypes = pd.concat([x.dtypes, meta.dtypes], axis=1)
+        bad = [(col, a, b) for col, a, b in dtypes.fillna('-').itertuples()
+               if not equal_dtypes(a, b)]
+        if not bad:
+            return x
+        errmsg = ("Partition type: `%s`\n%s" %
+                  (type(meta).__name__,
+                   asciitable(['Column', 'Found', 'Expected'], bad)))
+    else:
+        if equal_dtypes(x.dtype, meta.dtype):
+            return x
+        errmsg = ("Partition type: `%s`\n%s" %
+                  (type(meta).__name__,
+                   asciitable(['', 'dtype'], [('Found', x.dtype),
+                                              ('Expected', meta.dtype)])))
+
+    raise ValueError("Metadata mismatch found%s.\n\n"
+                     "%s" % ((" in `%s`" % funcname if funcname else ""),
+                             errmsg))
+
+
 ###############################################################
 # Testing
 ###############################################################
@@ -537,9 +603,8 @@ def assert_dask_graph(dask, label):
             k = k[0]
         if k.startswith(label):
             return True
-    else:
-        msg = "given dask graph doesn't contan label: {0}"
-        raise AssertionError(msg.format(label))
+    raise AssertionError("given dask graph doesn't contain label: {label}"
+                         .format(label=label))
 
 
 def assert_divisions(ddf):
@@ -550,9 +615,11 @@ def assert_divisions(ddf):
     if not ddf.known_divisions:
         return
 
-    index = lambda x: x if isinstance(x, pd.Index) else x.index
+    def index(x):
+        return (x if isinstance(x, pd.Index)
+                else x.index.get_level_values(0))
 
-    results = get_sync(ddf.dask, ddf._keys())
+    results = get_sync(ddf.dask, ddf.__dask_keys__())
     for i, df in enumerate(results[:-1]):
         if len(df):
             assert index(df).min() >= ddf.divisions[i]
