@@ -5,16 +5,17 @@ import operator
 import uuid
 
 try:
-    from cytoolz import curry, first
+    from cytoolz import curry, first, pluck
 except ImportError:
-    from toolz import curry, first
+    from toolz import curry, first, pluck
 
-from . import base, threaded
+from . import threaded
+from .base import Base, is_dask_collection, dont_optimize
+from .base import tokenize as _tokenize
 from .compatibility import apply
 from .core import quote
-from .context import _globals, defer_to_globals
-from .optimize import dont_optimize
-from .utils import concrete, funcname, methodcaller, ensure_dict
+from .context import _globals, globalmethod
+from .utils import funcname, methodcaller, OperatorMethodMixin
 from . import sharedict
 
 __all__ = ['Delayed', 'delayed']
@@ -65,11 +66,13 @@ def to_task_dask(expr):
     if isinstance(expr, Delayed):
         return expr.key, expr.dask
 
-    if isinstance(expr, base.Base):
+    if is_dask_collection(expr):
         name = 'finalize-' + tokenize(expr, pure=True)
-        keys = expr._keys()
-        dsk = expr._optimize(ensure_dict(expr.dask), keys)
-        dsk[name] = (expr._finalize, (concrete, keys))
+        keys = expr.__dask_keys__()
+        opt = getattr(expr, '__dask_optimize__', dont_optimize)
+        finalize, args = expr.__dask_postcompute__()
+        dsk = {name: (finalize, keys) + args}
+        dsk.update(opt(expr.__dask_graph__(), keys))
         return name, dsk
 
     if isinstance(expr, Iterator):
@@ -111,7 +114,7 @@ def tokenize(*args, **kwargs):
         pure = _globals.get('delayed_pure', False)
 
     if pure:
-        return base.tokenize(*args, **kwargs)
+        return _tokenize(*args, **kwargs)
     else:
         return str(uuid.uuid4())
 
@@ -301,7 +304,7 @@ def delayed(obj, name=None, pure=None, nout=None, traverse=True):
     if isinstance(obj, Delayed):
         return obj
 
-    if isinstance(obj, base.Base) or traverse:
+    if is_dask_collection(obj) or traverse:
         task, dsk = to_task_dask(obj)
     else:
         task = quote(obj)
@@ -333,18 +336,16 @@ def right(method):
     return _inner
 
 
-optimize = defer_to_globals('delayed_optimize', falsey=dont_optimize)(dont_optimize)
+def rebuild(dsk, key, length):
+    return Delayed(key, dsk, length)
 
 
-class Delayed(base.Base):
+class Delayed(Base, OperatorMethodMixin):
     """Represents a value to be computed by dask.
 
     Equivalent to the output from a single key in a dask graph.
     """
     __slots__ = ('_key', 'dask', '_length')
-    _finalize = staticmethod(first)
-    _default_get = staticmethod(threaded.get)
-    _optimize = staticmethod(optimize)
 
     def __init__(self, key, dsk, length=None):
         self._key = key
@@ -353,6 +354,24 @@ class Delayed(base.Base):
         self.dask = dsk
         self._length = length
 
+    def __dask_graph__(self):
+        return self.dask
+
+    def __dask_keys__(self):
+        return [self.key]
+
+    def __dask_tokenize__(self):
+        return self.key
+
+    __dask_scheduler__ = staticmethod(threaded.get)
+    __dask_optimize__ = globalmethod(dont_optimize, key='delayed_optimize')
+
+    def __dask_postcompute__(self):
+        return first, ()
+
+    def __dask_postpersist__(self):
+        return rebuild, (self._key, getattr(self, '_length', None))
+
     def __getstate__(self):
         return tuple(getattr(self, i) for i in self.__slots__)
 
@@ -364,9 +383,6 @@ class Delayed(base.Base):
     def key(self):
         return self._key
 
-    def _keys(self):
-        return [self.key]
-
     def __repr__(self):
         return "Delayed({0})".format(repr(self.key))
 
@@ -434,8 +450,18 @@ def call_function(func, func_token, args, kwargs, pure=None, nout=None):
     else:
         name = dask_key_name
 
-    args, dasks = unzip(map(to_task_dask, args), 2)
-    dsk = sharedict.merge(*dasks)
+    dsk = sharedict.ShareDict()
+    args_dasks = list(map(to_task_dask, args))
+    for arg, d in args_dasks:
+        if isinstance(d, sharedict.ShareDict):
+            dsk.update_with_key(d)
+        elif isinstance(arg, (str, tuple)):
+            dsk.update_with_key(d, key=arg)
+        else:
+            dsk.update(d)
+
+    args = tuple(pluck(0, args_dasks))
+
     if kwargs:
         dask_kwargs, dsk2 = to_task_dask(kwargs)
         dsk.update(dsk2)
@@ -490,6 +516,3 @@ for op in [operator.abs, operator.neg, operator.pos, operator.invert,
            operator.eq, operator.ge, operator.gt, operator.ne, operator.le,
            operator.lt, operator.getitem]:
     Delayed._bind_operator(op)
-
-
-base.normalize_token.register(Delayed, lambda a: a.key)
