@@ -1,4 +1,4 @@
-from dask.context import set_options, _globals, defer_to_globals
+from dask.context import set_options, _globals, globalmethod
 import dask.array as da
 import dask
 
@@ -36,21 +36,37 @@ def test_set_options_context_manger():
         del _globals['foo']
 
 
-def test_defer_to_globals():
-    @defer_to_globals('f')
+def foo():
+    return 'foo'
+
+
+def bar():
+    return 'bar'
+
+
+class Foo(object):
+    @globalmethod(key='f')
     def f():
         return 1
 
-    assert f() == 1
+    g = globalmethod(foo, key='g', falsey=bar)
+
+
+def test_globalmethod():
+    x = Foo()
+
+    assert x.f() == 1
 
     with dask.set_options(f=lambda: 2):
-        assert f() == 2
-    assert f() == 1
+        assert x.f() == 2
 
-    @defer_to_globals('f', falsey=lambda: 3)
-    def f():
-        return 1
+    with dask.set_options(f=foo):
+        assert x.f is foo
+        assert x.f() == 'foo'
 
-    with dask.set_options(f=None):
-        assert f() == 3
-    assert f() == 1
+    assert x.g is foo
+    assert x.g() == 'foo'
+
+    with dask.set_options(g=False):
+        assert x.g is bar
+        assert x.g() == 'bar'
