@@ -1,42 +1,78 @@
-from collections import Iterator, namedtuple
+from collections import namedtuple
 from operator import add, setitem
 import pickle
 from random import random
 
-from toolz import identity, partial
+from toolz import identity, partial, merge
 import pytest
 
+import dask
+from dask import set_options, compute
 from dask.compatibility import PY2, PY3
-from dask.delayed import delayed, to_task_dasks, compute, Delayed
+from dask.delayed import delayed, to_task_dask, Delayed
+from dask.utils_test import inc
 
 
-def test_to_task_dasks():
+class Tuple(object):
+    __dask_scheduler__ = staticmethod(dask.threaded.get)
+
+    def __init__(self, dsk, keys):
+        self._dask = dsk
+        self._keys = keys
+
+    def __dask_tokenize__(self):
+        return self._keys
+
+    def __dask_graph__(self):
+        return self._dask
+
+    def __dask_keys__(self):
+        return self._keys
+
+    def __dask_postcompute__(self):
+        return tuple, ()
+
+
+def test_to_task_dask():
     a = delayed(1, name='a')
     b = delayed(2, name='b')
-    task, dasks = to_task_dasks([a, b, 3])
+    task, dask = to_task_dask([a, b, 3])
     assert task == ['a', 'b', 3]
-    assert len(dasks) == 2
-    assert a.dask in dasks
-    assert b.dask in dasks
 
-    task, dasks = to_task_dasks((a, b, 3))
+    task, dask = to_task_dask((a, b, 3))
     assert task == (tuple, ['a', 'b', 3])
-    assert len(dasks) == 2
-    assert a.dask in dasks
-    assert b.dask in dasks
+    assert dict(dask) == merge(a.dask, b.dask)
 
-    task, dasks = to_task_dasks({a: 1, b: 2})
+    task, dask = to_task_dask({a: 1, b: 2})
     assert (task == (dict, [['b', 2], ['a', 1]]) or
             task == (dict, [['a', 1], ['b', 2]]))
-    assert len(dasks) == 2
-    assert a.dask in dasks
-    assert b.dask in dasks
+    assert dict(dask) == merge(a.dask, b.dask)
 
     f = namedtuple('f', ['x', 'y'])
     x = f(1, 2)
-    task, dasks = to_task_dasks(x)
+    task, dask = to_task_dask(x)
     assert task == x
-    assert dasks == []
+    assert dict(dask) == {}
+
+    task, dask = to_task_dask(slice(a, b, 3))
+    assert task == (slice, 'a', 'b', 3)
+    assert dict(dask) == merge(a.dask, b.dask)
+
+    # Issue https://github.com/dask/dask/issues/2107
+    class MyClass(dict):
+        pass
+
+    task, dask = to_task_dask(MyClass())
+    assert type(task) is MyClass
+    assert dict(dask) == {}
+
+    # Custom dask objects
+    x = Tuple({'a': 1, 'b': 2, 'c': (add, 'a', 'b')}, ['a', 'b', 'c'])
+    task, dask = to_task_dask(x)
+    assert task in dask
+    f = dask.pop(task)
+    assert f == (tuple, ['a', 'b', 'c'])
+    assert dask == x._dask
 
 
 def test_delayed():
@@ -56,6 +92,8 @@ def test_operators():
     a = delayed([1, 2, 3])
     assert a[0].compute() == 1
     assert (a + a).compute() == [1, 2, 3, 1, 2, 3]
+    b = delayed(2)
+    assert a[:b].compute() == [1, 2]
 
     a = delayed(10)
     assert (a + 1).compute() == 11
@@ -76,17 +114,17 @@ def test_methods():
 
 def test_attributes():
     a = delayed(2 + 1j)
+    assert a.real._key == a.real._key
     assert a.real.compute() == 2
     assert a.imag.compute() == 1
     assert (a.real + a.imag).compute() == 3
 
 
-def test_method_getattr_optimize():
+def test_method_getattr_call_same_task():
     a = delayed([1, 2, 3])
     o = a.index(1)
-    dsk = o._optimize(o.dask, o._keys())
     # Don't getattr the method, then call in separate task
-    assert getattr not in set(v[0] for v in dsk.values())
+    assert getattr not in set(v[0] for v in o.__dask_graph__().values())
 
 
 def test_delayed_errors():
@@ -103,15 +141,6 @@ def test_delayed_errors():
     pytest.raises(TypeError, lambda: bool(a))
 
 
-def test_compute():
-    a = delayed(1) + 5
-    b = a + 1
-    c = a + 2
-    assert compute(b, c) == (7, 8)
-    assert compute(b) == (7,)
-    assert compute([a, b], c) == ([6, 7], 8)
-
-
 def test_common_subexpressions():
     a = delayed([1, 2, 3])
     res = a[0] + a[0]
@@ -160,7 +189,6 @@ def test_lists_are_concrete():
     assert c.compute() == 20
 
 
-@pytest.mark.xfail
 def test_iterators():
     a = delayed(1)
     b = delayed(2)
@@ -169,13 +197,47 @@ def test_iterators():
     assert c.compute() == 3
 
     def f(seq):
-        assert isinstance(seq, Iterator)
         return sum(seq)
 
     c = delayed(f)(iter([a, b]))
     assert c.compute() == 3
 
 
+def test_traverse_false():
+    # Create a list with a dask value, and test that it's not computed
+    def fail(*args):
+        raise ValueError("shouldn't have computed")
+
+    a = delayed(fail)()
+
+    # list
+    x = [a, 1, 2, 3]
+    res = delayed(x, traverse=False).compute()
+    assert len(res) == 4
+    assert res[0] is a
+    assert res[1:] == x[1:]
+
+    # tuple that looks like a task
+    x = (fail, a, (fail, a))
+    res = delayed(x, traverse=False).compute()
+    assert isinstance(res, tuple)
+    assert res[0] == fail
+    assert res[1] is a
+
+    # list containing task-like-things
+    x = [1, (fail, a), a]
+    res = delayed(x, traverse=False).compute()
+    assert isinstance(res, list)
+    assert res[0] == 1
+    assert res[1][0] == fail and res[1][1] is a
+    assert res[2] is a
+
+    # traverse=False still hits top level
+    b = delayed(1)
+    x = delayed(b, traverse=False)
+    assert x.compute() == 1
+
+
 def test_pure():
     v1 = delayed(add, pure=True)(1, 2)
     v2 = delayed(add, pure=True)(1, 2)
@@ -185,6 +247,46 @@ def test_pure():
     assert myrand().key != myrand().key
 
 
+def test_pure_global_setting():
+    # delayed functions
+    func = delayed(add)
+
+    with set_options(delayed_pure=True):
+        assert func(1, 2).key == func(1, 2).key
+
+    with set_options(delayed_pure=False):
+        assert func(1, 2).key != func(1, 2).key
+
+    func = delayed(add, pure=True)
+    with set_options(delayed_pure=False):
+        assert func(1, 2).key == func(1, 2).key
+
+    # delayed objects
+    assert delayed(1).key != delayed(1).key
+    with set_options(delayed_pure=True):
+        assert delayed(1).key == delayed(1).key
+
+    with set_options(delayed_pure=False):
+        assert delayed(1, pure=True).key == delayed(1, pure=True).key
+
+    # delayed methods
+    data = delayed([1, 2, 3])
+    assert data.index(1).key != data.index(1).key
+
+    with set_options(delayed_pure=True):
+        assert data.index(1).key == data.index(1).key
+        assert data.index(1, pure=False).key != data.index(1, pure=False).key
+
+    with set_options(delayed_pure=False):
+        assert data.index(1, pure=True).key == data.index(1, pure=True).key
+
+    # magic methods always pure
+    with set_options(delayed_pure=False):
+        assert data.index.key == data.index.key
+        element = data[0]
+        assert (element + element).key == (element + element).key
+
+
 def test_nout():
     func = delayed(lambda x: (x, -x), nout=2, pure=True)
     x = func(1)
@@ -199,12 +301,25 @@ def test_nout():
     pytest.raises(ValueError, lambda: delayed(add, nout=-1))
     pytest.raises(ValueError, lambda: delayed(add, nout=True))
 
-    func = delayed(add, nout=1)
+    func = delayed(add, nout=None)
     a = func(1)
     assert a._length is None
     pytest.raises(TypeError, lambda: list(a))
     pytest.raises(TypeError, lambda: len(a))
 
+    func = delayed(lambda x: (x,), nout=1, pure=True)
+    x = func(1)
+    assert len(x) == 1
+    a, = x
+    assert a.compute() == 1
+    assert a._length is None
+    pytest.raises(TypeError, lambda: len(a))
+
+    func = delayed(lambda x: tuple(), nout=0, pure=True)
+    x = func(1)
+    assert len(x) == 0
+    assert x.compute() == tuple()
+
 
 def test_kwargs():
     def mysum(a, b, c=(), **kwargs):
@@ -222,6 +337,15 @@ def test_kwargs():
     assert dmysum(1, 2, c=c, four=4).key != dmysum(2, 2, c=c, four=4).key
 
 
+def test_custom_delayed():
+    x = Tuple({'a': 1, 'b': 2, 'c': (add, 'a', 'b')}, ['a', 'b', 'c'])
+    x2 = delayed(add, pure=True)(x, (4, 5, 6))
+    n = delayed(len, pure=True)(x)
+    assert delayed(len, pure=True)(x).key == n.key
+    assert x2.compute() == (1, 2, 3, 4, 5, 6)
+    assert compute(n, x2, x) == (3, (1, 2, 3, 4, 5, 6), (1, 2, 3))
+
+
 def test_array_delayed():
     np = pytest.importorskip('numpy')
     da = pytest.importorskip('dask.array')
@@ -234,14 +358,16 @@ def test_array_delayed():
     assert val.sum().compute() == (arr + arr + 1).sum()
     assert val[0, 0].compute() == (arr + arr + 1)[0, 0]
 
-    task, dasks = to_task_dasks(darr)
-    assert len(dasks) == 1
+    task, dsk = to_task_dask(darr)
     orig = set(darr.dask)
-    final = set(dasks[0])
+    final = set(dsk)
     assert orig.issubset(final)
     diff = final.difference(orig)
     assert len(diff) == 1
 
+    delayed_arr = delayed(darr)
+    assert (delayed_arr.compute() == arr).all()
+
 
 def test_array_bag_delayed():
     db = pytest.importorskip('dask.bag')
@@ -319,22 +445,22 @@ def test_callable_obj():
     assert f().compute() == 2
 
 
-def test_name_consitent_across_instances():
+def test_name_consistent_across_instances():
     func = delayed(identity, pure=True)
 
     data = {'x': 1, 'y': 25, 'z': [1, 2, 3]}
     if PY2:
-        assert func(data)._key == 'identity-777036d61a8334229dc0eda4454830d7'
+        assert func(data)._key == 'identity-6700b857eea9a7d3079762c9a253ffbd'
     if PY3:
-        assert func(data)._key == 'identity-1de4057b4cfa0ba7faed76b9c383cc99'
+        assert func(data)._key == 'identity-84c5e2194036c17d1d97c4e3a2b90482'
 
     data = {'x': 1, 1: 'x'}
     assert func(data)._key == func(data)._key
 
     if PY2:
-        assert func(1)._key == 'identity-d3eda9ebeead15c7e491960e89605b7f'
+        assert func(1)._key == 'identity-91f02358e13dca18cde218a63fee436a'
     if PY3:
-        assert func(1)._key == 'identity-5390b9efe3ddb6ea0557139003eef253'
+        assert func(1)._key == 'identity-7126728842461bf3d2caecf7b954fa3b'
 
 
 def test_sensitive_to_partials():
@@ -354,7 +480,8 @@ def test_delayed_name():
 
 
 def test_finalize_name():
-    import dask.array as da
+    da = pytest.importorskip('dask.array')
+
     x = da.ones(10, chunks=5)
     v = delayed([x])
     assert set(x.dask).issubset(v.dask)
@@ -365,3 +492,13 @@ def test_finalize_name():
         return s.split('-')[0]
 
     assert all(key(k).isalpha() for k in v.dask)
+
+
+def test_keys_from_array():
+    da = pytest.importorskip('dask.array')
+    from dask.array.utils import _check_dsk
+
+    X = da.ones((10, 10), chunks=5).to_delayed().flatten()
+    xs = [delayed(inc)(x) for x in X]
+
+    _check_dsk(xs[0].dask)
