@@ -13,11 +13,15 @@ else:
     pytestmark = pytest.mark.skipif(True,
                                     reason="graphviz exception with Python -OO flag")
 
+from dask import delayed
 from dask.utils import ensure_not_exists
 from IPython.display import Image, SVG
 
+
 # Since graphviz doesn't store a graph, we need to parse the output
 label_re = re.compile('.*\[label=(.*?) shape=.*\]')
+
+
 def get_label(line):
     m = label_re.match(line)
     if m:
@@ -70,9 +74,12 @@ def test_to_graphviz():
     assert set(labels).difference(dsk) == funcs
     assert set(labels).difference(funcs) == set(dsk)
 
+
 def test_to_graphviz_attributes():
     assert to_graphviz(dsk).graph_attr['rankdir'] == 'BT'
     assert to_graphviz(dsk, rankdir='LR').graph_attr['rankdir'] == 'LR'
+    assert to_graphviz(dsk, node_attr={'color': 'white'}).node_attr['color'] == 'white'
+    assert to_graphviz(dsk, edge_attr={'color': 'white'}).edge_attr['color'] == 'white'
 
 
 def test_aliases():
@@ -161,3 +168,13 @@ def test_filenames_and_formats():
         assert os.path.isfile(target)
         assert isinstance(result, expected_result_type)
         ensure_not_exists(target)
+
+
+def test_delayed_kwargs_apply():
+    def f(x, y=True):
+        return x + y
+
+    x = delayed(f)(1, y=2)
+    label = task_label(x.dask[x.key])
+    assert 'f' in label
+    assert 'apply' not in label
