@@ -1,5 +1,6 @@
-import pickle
 import functools
+import operator
+import pickle
 
 import numpy as np
 import pytest
@@ -8,7 +9,7 @@ from dask.sharedict import ShareDict
 from dask.utils import (takes_multiple_arguments, Dispatch, random_state_data,
                         memory_repr, methodcaller, M, skip_doctest,
                         SerializableLock, funcname, ndeepmap, ensure_dict,
-                        package_of, extra_titles)
+                        extra_titles, asciitable, itemgetter, partial_by_order)
 from dask.utils_test import inc
 
 
@@ -170,6 +171,22 @@ def test_extra_titles():
     assert extra_titles(example) == expected
 
 
+def test_asciitable():
+    res = asciitable(['fruit', 'color'],
+                     [('apple', 'red'),
+                      ('banana', 'yellow'),
+                      ('tomato', 'red'),
+                      ('pear', 'green')])
+    assert res == ('+--------+--------+\n'
+                   '| fruit  | color  |\n'
+                   '+--------+--------+\n'
+                   '| apple  | red    |\n'
+                   '| banana | yellow |\n'
+                   '| tomato | red    |\n'
+                   '| pear   | green  |\n'
+                   '+--------+--------+')
+
+
 def test_SerializableLock():
     a = SerializableLock()
     b = SerializableLock()
@@ -288,12 +305,14 @@ def test_ensure_dict():
     assert ensure_dict(md) == d
 
 
-def test_package_of():
-    import math
-    assert package_of(math.sin) is math
-    try:
-        import numpy
-    except ImportError:
-        pass
-    else:
-        assert package_of(numpy.memmap) is numpy
+def test_itemgetter():
+    data = [1, 2, 3]
+    g = itemgetter(1)
+    assert g(data) == 2
+    g2 = pickle.loads(pickle.dumps(g))
+    assert g2(data) == 2
+    assert g2.index == 1
+
+
+def test_partial_by_order():
+    assert partial_by_order(5, function=operator.add, other=[(1, 20)]) == 25
