@@ -1,16 +1,17 @@
 """
 A threaded shared-memory scheduler
 
-See async.py
+See scheduler.py
 """
 from __future__ import absolute_import, division, print_function
 
+import sys
 from collections import defaultdict
 from multiprocessing.pool import ThreadPool
 import threading
 from threading import current_thread, Lock
 
-from .async import get_async
+from .local import get_async
 from .context import _globals
 from .utils_test import inc, add  # noqa: F401
 
@@ -19,14 +20,16 @@ def _thread_get_id():
     return current_thread().ident
 
 
-default_pool = ThreadPool()
-default_thread = _thread_get_id()
 main_thread = current_thread()
-
+default_pool = None
 pools = defaultdict(dict)
 pools_lock = Lock()
 
 
+def pack_exception(e, dumps):
+    return e, sys.exc_info()[2]
+
+
 def get(dsk, result, cache=None, num_workers=None, **kwargs):
     """ Threaded cached implementation of dask.get
 
@@ -51,12 +54,15 @@ def get(dsk, result, cache=None, num_workers=None, **kwargs):
     >>> get(dsk, ['w', 'y'])
     (4, 2)
     """
+    global default_pool
     pool = _globals['pool']
     thread = current_thread()
 
     with pools_lock:
         if pool is None:
             if num_workers is None and thread is main_thread:
+                if default_pool is None:
+                    default_pool = ThreadPool()
                 pool = default_pool
             elif thread in pools and num_workers in pools[thread]:
                 pool = pools[thread][num_workers]
@@ -66,7 +72,7 @@ def get(dsk, result, cache=None, num_workers=None, **kwargs):
 
     results = get_async(pool.apply_async, len(pool._pool), dsk, result,
                         cache=cache, get_id=_thread_get_id,
-                        **kwargs)
+                        pack_exception=pack_exception, **kwargs)
 
     # Cleanup pools associated to dead threads
     with pools_lock:
